# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Tazminat Sorgula

Hitap üzerinden personelin tazminat bilgilerinin sorgulamasını yapar.

"""

from ulakbus.services.personel.hitap.hitap_sorgula import HITAPSorgula


class HizmetTazminatGetir(HITAPSorgula):
    """
    HITAP Sorgulama servisinden kalıtılmış Tazminat Bilgisi Sorgulama servisi

    """

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            bean_name (str): Hitap'tan gelen bean nesnesinin adı
            service_dict (dict): Hitap servisinden gelen kayıtların alanları,
                    ``HizmetTazminat`` modelinin alanlarıyla eşlenmektedir.
                    Servis tarafında gerekli olan alanlar listede tutulmaktadır.

        """

        self.service_name = 'HizmetTazminatSorgula'
        self.bean_name = 'HizmetTazminatServisBean'
        self.service_dict = {
            'fields': {
                'tckn': 'tckn',
                'kayit_no': 'kayitNo',
                'unvan_kod': 'unvanKod',
                'makam': 'makam',
                'gorev': 'gorev',
                'temsil': 'temsil',
                'tazminat_tarihi': 'tazminatTarihi',
                'tazminat_bitis_tarihi': 'tazminatBitisTarihi',
                'kadrosuzluk': 'kadrosuzluk',
                'kurum_onay_tarihi': 'kurumOnayTarihi',
            },
            'date_filter': ['tazminat_tarihi', 'tazminat_bitis_tarihi', 'kurum_onay_tarihi'],
            'required_fields': ['tckn']
        }
        super(HizmetTazminatGetir, self).handle()



# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""Bina ve Oda Modülü

Bu modül Ulakbus uygulaması için üniversite yerleşim modellerini içerir. Modellerin
amacı Üniversitelere ait yerleşke, bina ve oda bilgilerinin ilişkileri ile birlikte saklanmasıdır.

Campus, Building, RoomType, Room ve RoomFeature modellerinden oluşur.

"""

from pyoko import field, Model, ListNode
from ulakbus.models import Unit

__author__ = 'Ali Riza Keles'


class Campus(Model):
    """Kampüs Modeli

    Üniversite kampüslerine ait data modeli. Kampus adı ve koordinat bilgilerini içerir.

    Kampüs koordinatları lokasyon bazlı hesaplamalar için kullanılacaktır. Özellikle
    Unitime ile ders programı hazırlarken farklı lokasyonlar arası zaman hesaplamalarında
    kullanılmaktadır.

    """

    code = field.String("Kod", index=True)
    name = field.String("İsim", index=True)
    coordinate_x = field.String("X Koordinatı", index=True)
    coordinate_y = field.String("Y Koordinatı", index=True)

    class Meta:
        verbose_name = "Yerleşke"
        verbose_name_plural = "Yerleşkeler"
        search_fields = ['code', 'name']
        list_fields = ['code', 'name']

    def __unicode__(self):
        return '%s %s %s' % (self.code, self.name, self.coordinates())

    def coordinates(self):
        """Koordinatlar

        Returns:
            x ve y koordinatlarını birlikte döndürür.

        """
        return '%s %s' % (self.coordinate_x, self.coordinate_y)


class Building(Model):
    """Bina Modeli

    Universite kampuslerindeki binalara ait data modeli. Bina kod ve adının yanısıra
    koordinat bilgilerini içerir.

    Bina koordinatları lokasyon bazlı hesaplamalar için kullanılacaktır. Özellikle
    Unitime ile ders programı hazırlarken farklı lokasyonlar arası zaman hesaplamalarında
    kullanılmaktadır.

    """

    code = field.String("Kod", index=True)
    name = field.String("İsim", index=True)
    coordinate_x = field.String("X Koordinatı", index=True)
    coordinate_y = field.String("Y Koordinatı", index=True)
    campus = Campus()

    class Meta:
        verbose_name = "Bina"
        verbose_name_plural = "Binalar"
        search_fields = ['code', 'name']
        list_fields = ['code', 'name']

    def __unicode__(self):
        return '%s %s %s %s' % (self.code, self.name, self.coordinates(), self.campus)

    def coordinates(self):
        """"Koordinatlar

        Returns:
            x ve y koordınatlarını birlikte döndürür.
        """
        return '%s %s' % (self.coordinate_x, self.coordinate_y)


class RoomType(Model):
    """Oda tipleri modeli

    Odalar sinif, amfi, salon, kimya laboratuvari vb tiplere ayrilirlar. Ilgili notlarla
    birlikte bu tipler RoomType modelinde tanimlanir.

    """

    type = field.String("Oda Tipi", index=True)
    notes = field.Text("Notlar", index=True)

    class Meta:
        verbose_name = "Oda Tipi"
        verbose_name_plural = "Oda Tipleri"
        search_fields = ['type', 'notes']
        list_fields = ['type', ]

    def __unicode__(self):
        return '%s' % self.type


class Room(Model):
    """Oda modeli

    Üniversitenin sahip olduğu odalara (sınıf, lab, amfi) ait data modelidir. Her odanın
    bir kodu bulunur.

    Odalar, binalara ve binalar aracılığıyla kampüslere bağlanır.

    """

    code = field.String("Kod", index=True)
    name = field.String("İsim", index=True)
    room_type = RoomType("Oda Tipi", index=True)

    #: Bina içerisindeki kat bilgisi
    floor = field.String("Kat", index=True)
    capacity = field.Integer("Kapasite", index=True)
    building = Building()

    is_active = field.Boolean("Aktif", index=True)

    class Meta:
        verbose_name = "Oda"
        verbose_name_plural = "Odalar"
        search_fields = ['code', 'name']
        list_fields = ['code', 'name']

    class RoomDepartments(ListNode):
        """Oda Departman ListNode

        Bu odayı kullanabilecek birimlerin listesi saklanır.

        """

        unit = Unit()

    def __unicode__(self):
        return '%s %s %s' % (self.code, self.name, self.capacity)



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Ekle Servisi

Hitap ekle (insert) servislerinin kalıtılacağı
abstract HITAP Ekle servisini içeren modül.


Attributes:
    H_USER (str): Hitap kullanıcı adı
    H_PASS (str): Hitap kullanıcı şifresi

"""
__author__ = 'H.İbrahim Yılmaz (drlinux)'

from zato.server.service import Service
from ulakbus.services.personel.hitap.hitap_helper import HitapHelper
import os
import urllib2
from json import dumps
from six import iteritems

H_USER = os.environ["HITAP_USER"]
H_PASS = os.environ["HITAP_PASS"]


class HITAPEkle(Service):
    """
    Hitap Ekleme servislerinin kalıtılacağı abstract Zato servisi.

    Ekleme servisleri gerekli girdileri (Hitap dict, Hitap username, Hitap password)
    Hitap'a yollayıp dönecek cevabı elde edebilmektedirler.

    Attributes:
        service_name (str): İlgili Hitap sorgu servisinin adı
        service_dict (dict): Hitap servisine yollanacak datayı hazırlamak için sözlük.
            Servise gönderilecek verinin alanlarına ait sözlüğü
            ve tarih filtresi uygulanacak alanların listesini içerir.

    """

    def __init__(self):
        self.service_name = ''
        self.service_dict = {'fields': {}, 'date_filter': [], 'required_fields': []}
        super(HITAPEkle, self).__init__()

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Hitap'a gidecek isteği hazırlayacak ve gelen cevabı elde edecek olan
        request_json fonksiyonunu çağırır.

        """

        self.logger.info("zato service started to work.")
        conn = self.outgoing.soap['HITAP'].conn

        self.request_json(conn)

    def request_json(self, conn):
        """
        Connection bilgisi ve gerekli veriler ile Hitap'ın ilgili servisine
        istekte bulunup gelen cevabı uygun şekilde elde eder.

        Hitap'tan gelen verilerin sisteme uygun şekilde elde edilmesi için, gerekli
        Hitap sözlüğünü yaratan ve tarih alanı gibi belli alanlara uygun filteleri
        uygulayacak olan fonksiyonları çağırır.

        Veriler uygun şekilde elde edildikten sonra servis cevabı (payload) oluşturulur.

        Args:
            conn (zato.outgoing.soap.conn): Zato soap outgoing bağlantısı

        Raises:
            AttributeError: İlgili servis veya bean Hitap'ta bulunmayabilir.
            urllib2.URLError: Servis yanıt vermiyor.

        """

        status = "error"
        hitap_dict = []

        try:
            # connection for hitap
            with conn.client() as client:
                # hitap response
                service_call_name = self.service_mapper(self.service_name)

                if service_call_name:
                    request_data = client.factory.create(service_call_name)

                    if hasattr(request_data, 'kayitNo'):
                        del request_data.kayitNo
                    elif hasattr(request_data, 'ihzID'):
                        del request_data.ihzID

                    # filtering for some fields
                    if 'date_filter' in self.service_dict:
                        self.date_filter(hitap_dict)
                    self.custom_filter(hitap_dict)

                    if 'required_fields' in self.service_dict:
                        required_field_check = HitapHelper()
                        required_field_check.check_required_data(self.service_dict)

                    for dict_element in self.service_dict['fields']:
                        request_data[dict_element] = self.service_dict[dict_element]

                    service_name = self.service_name
                    hitap_service = getattr(client.service, self.service_name)(request_data,
                                                                               kullaniciAd=H_USER,
                                                                               sifre=H_PASS)
            status = "ok"

        except AttributeError:
            self.logger.exception("AttributeError")
            status = "error"

        except urllib2.URLError:
            self.logger.exception("Service unavailable!")
            status = "error"

        finally:
            self.response.payload = {'status': status,
                                     'result': self.create_hitap_json(hitap_service)}

    def create_hitap_json(self, hitap_service):
        """Ekleme servisinden dönen veriyi JSON formatına döndürür.
        Converts SOAP call result object into JSON

        """
        dict_result = dict((name, getattr(hitap_service, name)) for name in dir(hitap_service)
                           if not name.startswith('__'))

        self.logger.info("hitap_service json created.")

        return dumps(dict_result)

    def date_filter(self, hitap_dict):
        """
        Yerel kayıttaki tarih alanlarını HITAP servisine uygun biçime getirir.

        Hitap'ta tarih alanları için ``0001-01-01`` değeri boş değer anlamına gelirken,
        yerelde ``01.01.1900`` şeklindedir.

        Yerelde GG-AA-YYYY formatında tutulurken, HITAP servisi üzerindeki geçerli format YYYY-AA-GG
        şeklindedir.

        Args:
            hitap_dict (List[dict]): Hitap verisini yerele uygun biçimde tutan sözlük listesi

        """
        if hitap_dict['date_filter']:
            from datetime import datetime
            for record in hitap_dict:
                for field in hitap_dict['date_filter']:
                    if record[field] == "01.01.1900":
                        record[field] = '0001-01-01'
                    else:
                        date_format = datetime.strptime(record[field], "%d.%m.%Y")
                        record[field] = date_format.strftime("%Y-%m-%d")

    def custom_filter(self, hitap_dict):
        """
        Hitap sözlüğüne uygulanacak ek filtreleri (varsa) gerçekleştirir.

        Gerçekleştirimi bu servisten kalıtılan servislerde yapılmaktadır.

        Args:
            hitap_dict (List[dict]): Hitap verisini yerele uygun biçimde tutan sözlük listesi

        """
        pass

    def service_mapper(self, service_name):

        """Hitap'a yapılacak olan Ekleme çağrılarında gerekli datanın oluşmasını sağlamak için
        Servis ve ServisBean'lerini eşleyen method.

        """
        services_dict = {"HizmetAcikSureInsert": "ns1:HizmetAcikSureServisBean",
                         "HizmetAskerlikInsert": "ns1:HizmetAskerlikServisBean",
                         "HizmetBirlestirmeInsert": "ns1:HizmetBirlestirmeServisBean",
                         "HizmetBorclanmaInsert": "ns1:HizmetBorclanmaServisBean",
                         "HizmetCetvelInsert": "ns1:HizmetCetveliServisBean",
                         "HizmetIHSInsert": "ns1:HizmetIHSServisBean",
                         "HizmetKursInsert": "ns1:HizmetEgitimKursServisBean",
                         "HizmetMahkemeInsert": "ns1:HizmetMahkemeServisBean",
                         "HizmetNufusInsert": "ns1:HizmetNufusServisBean",
                         "HizmetOkulInsert": "ns1:HizmetEgitimOkulServisBean",
                         "HizmetTazminatInsert": "ns1:HizmetTazminatServisBean",
                         "HizmetUnvanInsert": "ns1:HizmetUnvanServisBean",
                         "hizmetIstisnaiIlgiInsert": "ns1:HizmetIstisnaiIlgiServisBean"}

        if service_name in services_dict:
            return services_dict[service_name]
        else:
            return False



# -*- coding:utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from ..personel import Personel
from pyoko import Model, field
import datetime
from .hitap_sebep import HitapSebep


class NufusKayitlari(Model):
    tckn = field.String("Sigortalının TC Kimlik No", index=True)
    ad = field.String("Adı", index=True)
    soyad = field.String("Soyadı", index=True)
    ilk_soy_ad = field.String("Memuriyete Girişteki İlk Soyadı", index=True)
    dogum_tarihi = field.Date("Doğum Tarihi", index=True, format="%d.%m.%Y")
    cinsiyet = field.String("Cinsiyet", index=True)
    emekli_sicil_no = field.Integer("Emekli Sicil No", index=True)
    memuriyet_baslama_tarihi = field.Date("Memuriyete İlk Başlama Tarihi", index=True,
                                          format="%d.%m.%Y")
    kurum_sicil = field.String("Kurum Sicili", index=True)
    maluliyet_kod = field.Integer("Malul Kod", index=True, choices="maluliyet_kod")
    yetki_seviyesi = field.String("Yetki Seviyesi", index=True)
    aciklama = field.String("Açıklama", index=True)
    kuruma_baslama_tarihi = field.Date("Kuruma Başlama Tarihi", index=True, format="%d.%m.%Y")
    gorev_tarihi_6495 = field.Date("Emeklilik Sonrası Göreve Başlama Tarihi", index=True,
                                   format="%d.%m.%Y")
    emekli_sicil_6495 = field.Integer("2. Emekli Sicil No", index=True)
    durum = field.Boolean("Durum", index=True)
    sebep = field.Integer("Sebep", index=True)
    sync = field.Integer("Senkronize", index=True)
    personel = Personel(one_to_one=True)

    # TODO: Personele gore unique olmali

    class Meta:
        app = 'Personel'
        verbose_name = "Nüfus Bilgisi"
        verbose_name_plural = "Nüfus Bilgileri"
        hitap_service_prefix = "HitapNufus"

    def __unicode__(self):
        return '%s %s %s' % (self.ad, self.soyad, self.emekli_sicil_no)


class HizmetKurs(Model):
    tckn = field.String("TC Kimlik No", index=True)
    kayit_no = field.String("Kursa Kayıt No", index=True)
    kurs_ogrenim_suresi = field.Integer("Kurs Öğrenim Süresi", index=True)
    mezuniyet_tarihi = field.Date("Mezuniyet Tarihi", index=True, format="%d.%m.%Y")
    kurs_nevi = field.Integer("Kurs Nevi", index=True, choices="kurs_nevi")
    bolum_ad = field.Integer("Bölüm Adı", index=True,
                             choices="bolum_adi")  # TODO: serviste karsiligi yok
    okul_ad = field.Integer("Okul Adı", index=True,
                            choices="okul_adi")  # TODO: servisten gelen string
    ogrenim_yeri = field.String("Öğrenim Yeri", index=True)
    denklik_tarihi = field.Date("Denklik Tarihi", index=True, format="%d.%m.%Y")
    denklik_okulu = field.String("Denklik Okulu", index=True)
    denklik_bolum = field.String("Denklik Bölüm", index=True)
    kurum_onay_tarihi = field.Date("Kurum Onay Tarihi", index=True, format="%d.%m.%Y")
    sync = field.Integer("Senkronize", index=True)
    personel = Personel()

    class Meta:
        app = 'Personel'
        verbose_name = "Kurs"
        verbose_name_plural = "Kurslar"
        list_fields = ['kayit_no', 'bolum_ad', 'okul_ad']
        search_fields = ['tckn', 'okul_ad', 'bolum_ad']
        hitap_service_prefix = "HitapKurs"

    def __unicode__(self):
        return '%s %s %s' % (self.kurs_nevi, self.bolum_ad, self.okul_ad)


class HizmetOkul(Model):
    kayit_no = field.String("Kayıt No", index=True)
    tckn = field.String("TC Kimlik No", index=True)
    ogrenim_durumu = field.Integer("Öğrenim Durumu", index=True, choices="ogrenim_durumu")
    mezuniyet_tarihi = field.Date("Mezuniyet Tarihi", index=True, format="%d.%m.%Y")
    okul_ad = field.String("Okul Adı", index=True)
    bolum = field.String("Bölüm", index=True)
    ogrenim_yeri = field.Integer("Öğrenim Yeri", index=True, choices="ogrenim_yeri")
    denklik_tarihi = field.Date("Denklik Tarihi", index=True, format="%d.%m.%Y")
    denklik_okul = field.String("Denklik Okul", index=True)
    denklik_bolum = field.String("Denklik Bölüm", index=True)
    ogrenim_suresi = field.Integer("Öğrenim Süresi", index=True)
    hazirlik = field.Integer("Hazırlık", index=True, choices="hazirlik_bilgisi")
    kurum_onay_tarihi = field.Date("Kurum Onay Tarihi", index=True, format="%d.%m.%Y")
    sync = field.Integer("Senkronize", index=True)
    personel = Personel()

    class Meta:
        app = 'Personel'
        verbose_name = "Okul"
        verbose_name_plural = "Okullar"
        list_fields = ['okul_ad', 'bolum', 'hazirlik', 'kayit_no']
        search_fields = ['okul_ad', 'bolum', 'tckn']
        hitap_service_prefix = "HitapOkul"

    def __unicode__(self):
        return '%s %s %s' % (self.kayit_no, self.bolum, self.okul_ad)


class HizmetMahkeme(Model):
    tckn = field.String("TC Kimlik No", index=True)
    kayit_no = field.String("Kayıt No", index=True)
    mahkeme_ad = field.String("Mahkeme Adı", index=True)
    sebep = field.Integer("Mahkeme Sebebi", index=True, choices="mahkeme_sebep")
    karar_tarihi = field.Date("Mahkeme Karar Tarihi", index=True, format="%d.%m.%Y")
    karar_sayisi = field.Integer("Karar Sayısı", index=True)
    kesinlesme_tarihi = field.Date("Kesinleşme Tarihi", index=True, format="%d.%m.%Y")
    asil_dogum_tarihi = field.Date("Asıl Doğum Tarihi", index=True, format="%d.%m.%Y")
    tashih_dogum_tarihi = field.Date("Tashih Doğum Tarihi", index=True, format="%d.%m.%Y")
    asil_ad = field.String("Asıl Ad", index=True)
    tashih_ad = field.String("Tashih Ad", index=True)
    asil_soyad = field.String("Asıl Soyad", index=True)
    tashih_soyad = field.String("Tashih Soyad", index=True)
    gecerli_dogum_tarihi = field.Date("Geçerli Doğum Tarihi", index=True, format="%d.%m.%Y")
    aciklama = field.String("Açıklama", index=True)
    gun_sayisi = field.Integer("Gün Sayısı", index=True)
    kurum_onay_tarihi = field.Date("Kurum Onay Tarihi", index=True, format="%d.%m.%Y")
    sync = field.Integer("Senkronize", index=True)
    personel = Personel()

    class Meta:
        app = 'Personel'
        verbose_name = "Mahkeme"
        verbose_name_plural = "Mahkemeler"
        list_fields = ['mahkeme_ad', 'karar_sayisi', 'aciklama', 'kurum_onay_tarihi']
        search_fields = ['kayit_no', 'mahkeme_ad', 'karar_sayisi']
        hitap_service_prefix = "HitapMahkeme"

    def __unicode__(self):
        return '%s %s %s' % (self.mahkeme_ad, self.karar_tarihi, self.aciklama)


class HizmetBirlestirme(Model):
    tckn = field.String("TC Kimlik No", index=True)
    kayit_no = field.String("Kayıt No", index=True)
    sgk_nevi = field.Integer("SGK Nevi", index=True, choices="sgk_nevi")
    sgk_sicil_no = field.String("SGK Sicil No", index=True)
    baslama_tarihi = field.Date("Başlama Tarihi", index=True, format="%d.%m.%Y")
    bitis_tarihi = field.Date("Bitiş Tarihi", index=True, format="%d.%m.%Y")
    sure = field.Integer("Süre", index=True)
    kamu_isyeri_ad = field.String("Kamu İşyeri Adı", index=True)
    ozel_isyeri_ad = field.String("Özel İşyeri Adı", index=True)
    bag_kur_meslek = field.String("Bağ-Kur Meslek", index=True)
    ulke_kod = field.Integer("Ülke Kodu", index=True)
    banka_sandik_kod = field.Integer("Banka Sandık Kodu", index=True, choices="banka_kod")
    kidem_tazminat_odeme_durumu = field.String("Kıdem Tazminat Ödeme Durumu", index=True,
                                               choices="kidem_tazminat_odeme_durumu")
    ayrilma_nedeni = field.String("Ayrılma Nedeni", index=True)
    kha_durum = field.Integer("KHA Durum", index=True, choices="kha_durum")
    kurum_onay_tarihi = field.Date("Kurum Onay Tarihi", index=True, format="%d.%m.%Y")
    sync = field.Integer("Senkronize", index=True)
    personel = Personel()

    class Meta:
        app = 'Personel'
        verbose_name = "Birleştirme"
        verbose_name_plural = "Birleştirmeler"
        list_fields = ['sgk_sicil_no', 'baslama_tarihi', 'bitis_tarihi', 'kamu_isyeri_ad']
        search_fields = ['kayit_no', 'sgk_sicil_no', 'kamu_isyeri_ad']
        hitap_service_prefix = "HitapBirlestirme"

    def __unicode__(self):
        return '%s %s' % (self.kayit_no, self.sgk_nevi)


class HizmetTazminat(Model):
    kayit_no = field.String("Kayıt No", index=True)
    tckn = field.String("TC Kimlik No", index=True)
    unvan_kod = field.Integer("Ünvan Kodu", index=True)
    makam = field.Integer("Makam", index=True)
    gorev = field.Integer("Görev", index=True)
    temsil = field.Integer("Temsil", index=True)
    tazminat_tarihi = field.Date("Tazminat Tarihi", index=True, format="%d.%m.%Y")
    tazminat_bitis_tarihi = field.Date("Tazminat Bitiş Tarihi", index=True, format="%d.%m.%Y")
    kadrosuzluk = field.Integer("Kadrosuzluk", index=True)
    kurum_onay_tarihi = field.Date("Kurum Onay Tarihi", index=True, format="%d.%m.%Y")
    sync = field.Integer("Senkronize", index=True)
    personel = Personel()

    class Meta:
        app = 'Personel'
        verbose_name = "Tazminat"
        verbose_name_plural = "Tazminatlar"
        list_fields = ['unvan_kod', 'makam', 'gorev']
        search_fields = ['makam', 'gorev', 'temsil']
        hitap_service_prefix = "HitapTazminat"

    def __unicode__(self):
        return '%s %s' % (self.gorev, self.tazminat_tarihi)


class HizmetUnvan(Model):
    kayit_no = field.String("Hizmet Kayıt No", index=True)
    tckn = field.String("TC Kimlik No", index=True)
    unvan_kod = field.Integer("Ünvan Kodu", index=True)
    unvan_tarihi = field.Date("Ünvan Tarihi", index=True, format="%d.%m.%Y")
    unvan_bitis_tarihi = field.Date("Ünvan Bitiş Tarihi", index=True, format="%d.%m.%Y")
    hizmet_sinifi = field.String("Hizmet Sınıfı", index=True)
    asil_vekil = field.String("Asıl Vekil", index=True)
    atama_sekli = field.String("Atama Sekli", index=True)
    kurum_onay_tarihi = field.Date("Kurum Onay Tarihi", index=True, format="%d.%m.%Y")
    fhz_orani = field.Float("FHZ Oranı", index=True)
    sync = field.Integer("Senkronize", index=True)
    personel = Personel()

    class Meta:
        app = 'Personel'
        verbose_name = "Ünvan"
        verbose_name_plural = "Ünvanlar"
        list_fields = ['unvan_kod', 'hizmet_sinifi', 'kurum_onay_tarihi']
        search_fields = ['unvan_kod', 'hizmet_sinifi']
        hitap_service_prefix = "HitapUnvan"

    def __unicode__(self):
        return '%s %s' % (self.unvan_kod, self.hizmet_sinifi)


class HizmetAcikSure(Model):
    tckn = field.String("TC Kimlik No", index=True)
    kayit_no = field.String("Kayıt No", index=True)
    acik_sekil = field.Integer("Açığa Alınma Şekli", index=True, choices="acik_sekli")
    iade_sekil = field.Integer("İade Şekil", index=True)
    hizmet_durum = field.Integer("Hizmet Durumu", index=True, choices="hizmet_durumu")
    husus = field.Integer("Husus", index=True, choices="husus")
    husus_aciklama = field.String("Husus Açıklaması", index=True)
    aciga_alinma_tarih = field.Date("Açığa Alınma Tarihi", index=True, format="%d.%m.%Y")
    goreve_son_tarih = field.Date("Göreve Son Tarih", index=True, format="%d.%m.%Y")
    goreve_iade_istem_tarih = field.Date("Göreve İade İstem Tarihi", index=True, format="%d.%m.%Y")
    goreve_iade_tarih = field.Date("Göreve İade Tarihi", index=True, format="%d.%m.%Y")
    acik_aylik_bas_tarih = field.Date("Açık Aylık Başlama Tarihi", index=True, format="%d.%m.%Y")
    acik_aylik_bit_tarih = field.Date("Açık Aylık Bitiş Tarihi", index=True, format="%d.%m.%Y")
    goreve_son_aylik_bas_tarih = field.Date("Göreve Sonlandırma Aylık Başlangıç Tarihi", index=True,
                                            format="%d.%m.%Y")
    goreve_son_aylik_bit_tarih = field.Date("Göreve Sonlandırma Aylık Bitiş Tarihi", index=True,
                                            format="%d.%m.%Y")
    s_yonetim_kald_tarih = field.Date("Sıkı Yönetim Kaldırıldığı Tarih", index=True,
                                      format="%d.%m.%Y")
    aciktan_atanma_tarih = field.Date("Açıktan Atanma Tarihi", index=True, format="%d.%m.%Y")
    kurum_onay_tarihi = field.Date("Kurum Onay Tarihi", index=True, format="%d.%m.%Y")
    sync = field.Integer("Senkronize", index=True)
    personel = Personel()

    class Meta:
        app = 'Personel'
        verbose_name = "Açığa Alınma"
        verbose_name_plural = "Açığa Alınmalar"
        list_fields = ['acik_sekil', 'aciga_alinma_tarih', 'kurum_onay_tarihi']
        search_fields = ['hizmet_durum', 'acik_sekil', 'aciga_alinma_tarih']
        hitap_service_prefix = "HitapAcikSure"

    def __unicode__(self):
        return '%s %s %s' % (self.iade_sekil, self.kayit_no, self.aciga_alinma_tarih)


class HizmetBorclanma(Model):
    tckn = field.String("TC Kimlik No", index=True)
    kayit_no = field.String("Kayıt No", index=True)
    ad = field.String("Ad", index=True)
    soyad = field.String("Soyad", index=True)
    emekli_sicil = field.String("Emekli Sicili", index=True)
    derece = field.Integer("Derece", index=True)
    kademe = field.Integer("Kademe", index=True)
    ekgosterge = field.Integer("Ek Gösterge", index=True)
    baslama_tarihi = field.Date("Başlama Tarihi", index=True, format="%d.%m.%Y")
    bitis_tarihi = field.Date("Bitiş Tarihi", index=True, format="%d.%m.%Y")
    gun_sayisi = field.Integer("Gün Sayısı", index=True)
    kanun_kod = field.Integer("Kanun Kodu", index=True, choices="kanun_kod")
    borc_nevi = field.Integer("Borç Nevi", index=True, choices="borc_nevi")
    toplam_tutar = field.Float("Toplam Tutar", index=True)
    odenen_miktar = field.Float("Ödenen Miktar", index=True)
    calistigi_kurum = field.String("çalıştığı Kurum", index=True)
    isyeri_il = field.String("İşyeri İli", index=True)
    isyeri_ilce = field.String("İşyeri İlçesi", index=True)

    borclanma_tarihi = field.Date("Borçlanma Tarihi", index=True, format="%d.%m.%Y")
    odeme_tarihi = field.Date("Ödeme Tarihi", index=True, format="%d.%m.%Y")
    kurum_onay_tarihi = field.Date("Kurum Onay Tarihi", index=True, format="%d.%m.%Y")
    sync = field.Integer("Senkronize", index=True)
    personel = Personel()

    class Meta:
        app = 'Personel'
        verbose_name = "Borçlanma"
        verbose_name_plural = "Borçlanmalar"
        list_fields = ['ad', 'soyad', 'toplam_tutar', 'odenen_miktar', 'borclanma_tarihi']
        search_fields = ['tckn', 'ad', 'soyad']
        hitap_service_prefix = "HitapBorclanma"

    def __unicode__(self):
        return '%s %s %s' % (self.borc_nevi, self.calistigi_kurum, self.gun_sayisi)


class HizmetIHS(Model):
    tckn = field.String("TC Kimlik No", index=True)
    kayit_no = field.String("Kayıt No", index=True)
    baslama_tarihi = field.Date("Başlama Tarihi", index=True, format="%d.%m.%Y")
    bitis_tarihi = field.Date("Bitiş Tarihi", index=True, format="%d.%m.%Y")
    ihz_nevi = field.Integer("İHZ Nevi", index=True)
    sync = field.Integer("Senkronize", index=True)
    personel = Personel()

    class Meta:
        app = 'Personel'
        verbose_name = "İtibari Hizmet Süresi"
        verbose_name_plural = "İtibari Hizmet Süreleri"
        list_fields = ['kayit_no', 'ihz_nevi']
        search_fields = ['tckn', 'ihz_nevi']
        hitap_service_prefix = "HitapIHS"

    def __unicode__(self):
        return '%s %s' % (self.baslama_tarihi, self.ihz_nevi)


class HizmetIstisnaiIlgi(Model):
    tckn = field.String("TC Kimlik No", index=True)
    kayit_no = field.String("Kayıt No", index=True)
    baslama_tarihi = field.Date("Başlama Tarihi", index=True, format="%d.%m.%Y")
    bitis_tarihi = field.Date("Bitiş Tarihi", index=True, format="%d.%m.%Y")
    gun_sayisi = field.Integer("Gün Sayısı", index=True)
    istisnai_ilgi_nevi = field.Integer("İstisnai İlgi Nevi", index=True)
    kha_durum = field.Integer("KHA Durum", index=True, choices="kha_durum")
    kurum_onay_tarihi = field.Date("Kurum Onay Tarihi", index=True, format="%d.%m.%Y")
    sync = field.Integer("Senkronize", index=True)
    personel = Personel()

    class Meta:
        app = 'Personel'
        verbose_name = "İstisnai İlgi"
        verbose_name_plural = "İstisnai İlgiler"
        list_fields = ['baslama_tarihi', 'bitis_tarihi', 'istisnai_ilgi_nevi', 'kha_durum']
        search_fields = ['istisnai_ilgi_nevi', 'kha_durum']
        hitap_service_prefix = "HitapIstisnaiIlgi"

    def __unicode__(self):
        return '%s %s %s' % (self.kayit_no, self.istisnai_ilgi_nevi, self.kha_durum)


class HizmetKayitlari(Model):
    tckn = field.String("TC Kimlik No", index=True)
    kayit_no = field.String("Kayıt No", index=True)
    baslama_tarihi = field.Date("Başlama Tarihi", index=True, format="%d.%m.%Y")
    bitis_tarihi = field.Date("Bitiş Tarihi", index=True, format="%d.%m.%Y")
    gorev = field.String("Görev", index=True)  # birim + kadro unvanı
    unvan_kod = field.Integer("Unvan Kod", index=True)  # kadro unvan kodu
    yevmiye = field.String("Yevmiye", index=True)
    ucret = field.String("Ücret", index=True)
    hizmet_sinifi = field.Integer("Hizmet Sınıfı", index=True,
                                  choices="hizmet_sinifi")  # atama modelinden gelecek
    kadro_derece = field.Integer("Kadro Derecesi", index=True)  # personelden gelecek
    odeme_derece = field.Integer("Ödeme Derecesi", index=True)  # personelden gelecek
    odeme_kademe = field.Integer("Ödeme Kademesi", index=True)  # personelden gelecek (gorunen)
    odeme_ekgosterge = field.Integer("Ödeme Ek Göstergesi", index=True)  # personelden gelecek
    kazanilmis_hak_ayligi_derece = field.Integer("Kazanılmış Hak Aylığı Derecesi",
                                                 index=True)  # personelden gelecek
    kazanilmis_hak_ayligi_kademe = field.Integer("Kazanılmış Hak Aylığı Kademesi",
                                                 index=True)  # personelden gelecek (gorunen)
    kazanilmis_hak_ayligi_ekgosterge = field.Integer("Kazanılmış Hak Aylığı Ek Göstergesi",
                                                     index=True)  # personelden gelecek
    emekli_derece = field.Integer("Emekli Derecesi", index=True)  # personelden gelecek
    emekli_kademe = field.Integer("Emekli Kademe", index=True)  # personelden gelecek (gorunen)
    emekli_ekgosterge = field.Integer("Emekli Ek Göstergesi", index=True)  # personelden gelecek
    sebep_kod = HitapSebep("Hitap Sebep Kodu")
    kurum_onay_tarihi = field.Date("Kurum Onay Tarihi", index=True, format="%d.%m.%Y")
    sync = field.Integer("Senkronize", index=True)
    personel = Personel()

    # hizmet cetveline birden cok modelden veri girilmektedir. bu alanda, hizmet
    # cetveli kaydini olusturan diger modeldeki kaydin key i saklanir.
    model_key = field.String()

    # post save metodunda baslangic ya da bitis tarihinden set edilir.
    order_date = field.DateTime()

    class Meta:
        app = 'Personel'
        verbose_name = "Kayıt"
        verbose_name_plural = "Kayıtlar"
        list_fields = ['unvan_kod', 'gorev', 'yevmiye', 'ucret', 'hizmet_sinifi']
        search_fields = ['unvan_kod', 'gorev', 'yevmiye']
        hitap_service_prefix = "HitapHizmetCetveli"

    def post_save(self):
        """
        Hizmet cetvelindeki kayıtların başlama veya bitiş tarihleri
        geliyor. Her kaydın başlama ve bitiş tarihi beraber gelmiyor.
        Bu yüzden kayıtların sıralanabilmesi için order_date alanı
        eklendi. Bitiş tarihi başka bir kaydın başlangıç tarihi ise
        önce bitiş tarihi olan kayıt gürüntülenmelidir. Bu yüzden
        başlangıç tarihine +1 saat eklendi.
        """
        if self.baslama_tarihi != datetime.date(1900, 1, 1):
            self.order_date = datetime.datetime.combine(self.baslama_tarihi, datetime.time(1))
        else:
            self.order_date = self.bitis_tarihi
        self.save()

    def post_creation(self):
        if self.personel:
            self.tckn = self.personel.tckn
            self.gorev = "%s %s" % (self.personel.birim.name, self.personel.kadro.unvan)
            self.unvan_kod = self.personel.kadro.unvan
            self.hizmet_sinifi = self.personel.atama.hizmet_sinifi
            self.kadro_derece = self.personel.kadro_derece
            self.odeme_derece = self.personel.gorev_ayligi_derece
            self.odeme_kademe = self.personel.gorunen_gorev_ayligi_kademe
            self.odeme_ekgosterge = self.personel.gorev_ayligi_ekgosterge
            self.kazanilmis_hak_ayligi_derece = self.personel.kazanilmis_hak_derece
            self.kazanilmis_hak_ayligi_kademe = self.personel.gorunen_kazanilmis_hak_kademe
            self.kazanilmis_hak_ayligi_ekgosterge = self.personel.kazanilmis_hak_ekgosterge
            self.emekli_derece = self.personel.emekli_muktesebat_derece
            self.emekli_kademe = self.personel.gorunen_emekli_muktesebat_kademe
            self.emekli_ekgosterge = self.personel.emekli_muktesebat_ekgosterge
            self.save()

    def __unicode__(self):
        return '%s %s %s' % (self.unvan_kod, self.hizmet_sinifi, self.gorev)


class AskerlikKayitlari(Model):
    askerlik_nevi = field.Integer("Askerlik Nevi", index=True, choices="askerlik_nevi")
    baslama_tarihi = field.Date("Başlama Tarihi", index=True, format="%d.%m.%Y")
    bitis_tarihi = field.Date("Bitiş Tarihi", index=True, format="%d.%m.%Y")
    kayit_no = field.String("Kayıt No", index=True)
    kita_baslama_tarihi = field.Date("Kıta Başlama Tarihi", index=True, format="%d.%m.%Y")
    kita_bitis_tarihi = field.Date("Kıta Bitiş Tarihi", index=True, format="%d.%m.%Y")
    muafiyet_neden = field.String("Muafiyet Neden", index=True)
    sayilmayan_gun_sayisi = field.Integer("Sayılmayan Gün Sayısı", index=True)
    sinif_okulu_sicil = field.String("Sınıf Okulu Sicil", index=True)
    subayliktan_erlige_gecis_tarihi = field.Date("Subaylıktan Erliğe Geçiş Tarihi", index=True,
                                                 format="%d.%m.%Y")
    subay_okulu_giris_tarihi = field.Date("Subay Okulu Giriş Tarihi", index=True, format="%d.%m.%Y")
    tckn = field.String("TC Kimlik No", index=True)
    tegmen_nasp_tarihi = field.Date("Teğmen Nasp Tarihi", index=True, format="%d.%m.%Y")
    gorev_yeri = field.String("Görev Yeri", index=True)
    kurum_onay_tarihi = field.Date("Kurum Onay Tarihi", index=True, format="%d.%m.%Y")
    astegmen_nasp_tarihi = field.Date("Asteğmen Nasp Tarihi", index=True, format="%d.%m.%Y")
    sync = field.Integer("Senkronize", index=True)
    personel = Personel()

    class Meta:
        app = 'Personel'
        verbose_name = "Kayıt"
        verbose_name_plural = "Kayıtlar"
        list_fields = ['askerlik_nevi', 'kita_baslama_tarihi', 'gorev_yeri']
        search_fields = ['askerlik_nevi', 'gorev_yeri', 'tckn']
        hitap_service_prefix = "HitapAskerlik"

    def __unicode__(self):
        return '%s %s %s %s' % (
            self.askerlik_nevi, self.kayit_no, self.kita_baslama_tarihi, self.gorev_yeri)


# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#
# Yeni Personel Ekle WF adimlarini icerir.
from pyoko.exceptions import ObjectDoesNotExist
from zengine.views.crud import CrudView
from zengine.forms import JsonForm, fields
from ulakbus.models.personel import Personel

"""
``button`` nesnelerinin stillendirilmesi için bu nesnelere ``style`` anahtarı ile css class'ları
gönderilebilir. Bu class'lar şunlar olabilir:
​
    - btn-default
    - btn-primary
    - btn-success
    - btn-info
    - btn warning
"""


# Views
class YeniPersonelEkle(CrudView):
    class Meta:
        model = 'Personel'

    def yeni_personel_form(self):
        """

        Yeni Personel eklenirken Tc Kimlik Numarası ekranı getirmek için kullanılan fonksiyon

        """
        _form = YeniPersonelTcknForm(current=self.current)
        self.form_out(_form)

    def mernis_kimlik_bilgileri_getir(self):
        tckn = self.current.input['form']['tckn']

        # Personelin daha önce bulunup bulunmadığını kontrol et
        try:
            Personel.objects.get(tckn=tckn)
            self.current.task_data['mernis_tamam'] = False
            self.current.task_data['hata_msg'] = "Personel Daha Önce Kaydedilmiş"
        except ObjectDoesNotExist:
            from ulakbus.services.zato_wrapper import MernisKimlikBilgileriGetir, \
                MernisCuzdanBilgileriGetir

            # Kimlik bilgileri mernis servisi üzerinden çekilecek
            mernis_bilgileri = MernisKimlikBilgileriGetir(tckn=str(tckn))
            response = mernis_bilgileri.zato_request()
            self.current.task_data['mernis_tamam'] = True
            self.current.task_data['kimlik_bilgileri'] = response
            # Cüzdan bilgileri mernis servisi üzerinden çekilecek
            mernis_bilgileri = MernisCuzdanBilgileriGetir(tckn=str(tckn))
            response = mernis_bilgileri.zato_request()
            self.current.task_data['cuzdan_tamam'] = True
            self.current.task_data['cuzdan_bilgileri'] = response
            self.current.task_data['tckn'] = tckn

            self.current.set_message(title='%s TC no için Mernis servisi başlatıldı' % tckn,
                                     msg='', typ=1, url="/wftoken/%s" % self.current.token)

    def mernis_adres_bilgileri_getir(self, tckn=None):
        if not tckn:
            tckn = self.current.input['form']['tckn']
        # Adres bilgileri mernis servisi üzerinden çekilecek
        from ulakbus.services.zato_wrapper import KPSAdresBilgileriGetir
        mernis_bilgileri = KPSAdresBilgileriGetir(tckn=str(tckn))
        response = mernis_bilgileri.zato_request()

        self.current.task_data['adres_tamam'] = True
        self.current.task_data['adres_bilgileri'] = response

        return response

    def yeni_ekle_kontrol(self):
        _form = IletisimveEngelliDurumBilgileriForm(current=self.current)
        self.current.output['msgbox'] = {
            'type': 'info', "title": 'Personel Kayıt Uyarısı',
            "msg": """
                   Lütfen Personel bilgilerinin doğruluğunu kontrol ediniz.
                   Kayıt işlemi geri döndürülemez.
                   """
        }

        _form.ikamet_adresi = self.current.task_data['adres_bilgileri']['ikamet_adresi']
        _form.ikamet_il = self.current.task_data['adres_bilgileri']['ikamet_il']
        _form.ikamet_ilce = self.current.task_data['adres_bilgileri']['ikamet_ilce']

        _form.kaydet = fields.Button("Kaydet (Bu işlem geri alınamaz!)", cmd="kaydet",
                                     btn='success')
        _form.iptal = fields.Button("İptal", cmd="iptal", flow="iptal", form_validation=False)

        kimlik_bilgileri = """**Adı**: {ad}
                           **Soyad**: {soyad}
                           **Doğum Tarihi**: {dogum_tarihi}
                           """.format(**self.current.task_data['kimlik_bilgileri'])

        adres_bilgileri = """**İl**: {ikamet_il}
                          **İlçe**: {ikamet_ilce}
                          **Adres**: {ikamet_adresi}
                          """.format(**self.current.task_data['adres_bilgileri'])

        output = [{'Kişi Bilgileri': kimlik_bilgileri, 'Adres Bilgileri': adres_bilgileri}]

        self.current.output['object'] = {
            "type": "table-multiRow",
            "title": "Kişi Bilgileri",
            "fields": output,
            "actions": False
        }
        self.form_out(_form)

    def kaydet(self):
        yeni_personel = Personel()
        yeni_personel.tckn = self.current.task_data['tckn']

        # Task data içinde gelen bilgiler birleştirilecek
        personel_data = {}
        personel_data.update(self.current.task_data['kimlik_bilgileri'])
        personel_data.update(self.current.task_data['cuzdan_bilgileri'])
        personel_data.update(self.current.task_data['adres_bilgileri'])

        for key in personel_data:
            setattr(yeni_personel, key, personel_data[key])

        yeni_personel.save()

        self.current.output['msgbox'] = {
            'type': 'success',
            "title": '%s %s Başarı İle Kaydedildi' % (yeni_personel.ad, yeni_personel.soyad),
            "msg": """
                   Personel Başarı ile Kaydedildi, Personele atama yapabilir veya daha sonra
                   atama yapmak için "İşlemi Bitir" Butonuna tıklayabilirsiniz
                   """
        }

        _form = JsonForm(current=self.current)

        # todo: bu buton ilgili personel secili olarak yeni bir wf baslatacak
        _form.geri = fields.Button("Atama Yap", style="btn-success", cmd="atama_yap")

        _form.iptal = fields.Button("İşlemi Bitir", cmd="bitir")
        self.form_out(_form)

        self.current.task_data['personel_id'] = yeni_personel.key

    def hata_goster(self):
        if self.current.task_data['hata_msg']:
            msg = self.current.task_data['hata_msg']
        else:
            msg = "Bilinmeyen bir hata oluştu :( sebebini biliyorsanız bizede söyleyinde düzeltelim"
        self.current.output['msgbox'] = {
            'type': 'error', "title": 'İşlem Başarısız',
            "msg": msg
        }

        _form = JsonForm(current=self.current)
        _form.geri = fields.Button("Yeni bir Tc Kimlik Numarası ile dene", style="btn-success",
                                   cmd="tekrar")
        _form.iptal = fields.Button("İptal", cmd="iptal")
        self.form_out(_form)


# Formlar
class YeniPersonelTcknForm(JsonForm):
    class Meta:
        title = 'Yeni Personel Ekle'
        help_text = "Kimlik Numarası ile MERNIS ve HITAP bilgileri getir."

    tckn = fields.String("TC No")
    cmd = fields.String("Bilgileri Getir", type="button")


class IletisimveEngelliDurumBilgileriForm(JsonForm):
    class Meta:
        title = 'Bilgi Kontrol Ekranı'
        help_text = "Yeni Personelin Iletisim Bilgilerini Duzenle."
        grouping = [
            {
                "layout": "6",
                "groups": [
                    {
                        "group_title": "Adres Bilgileri",
                        "items": ['ikamet_adresi', 'ikamet_il', 'ikamet_ilce',
                                  'adres_2', 'adres_2_posta_kodu'],
                        "collapse": True,
                    }
                ]
            },
            {
                "layout": "6",
                "groups": [
                    {
                        "group_title": "İletişim Bilgileri",
                        "items": ['oda_tel_no', 'cep_telefonu', 'e_posta', 'e_posta_2',
                                  'e_posta_3', 'web_sitesi', 'notlar'],
                        "collapse": True,
                    }
                ]
            },
            {
                "layout": "12",
                "groups": [
                    {
                        "group_title": "Engel Bilgileri",
                        "items": ['engelli_durumu', 'engel_grubu', 'engel_derecesi', 'engel_orani']
                    }
                ]
            }
        ]

    ikamet_adresi = fields.Text("İkamet Adresi")
    ikamet_il = fields.String("İkamet Il")
    ikamet_ilce = fields.String("İkamet Ilce")
    adres_2 = fields.String("Adres 2")
    adres_2_posta_kodu = fields.String("Adres 2 Posta Kodu")
    oda_tel_no = fields.String("Oda Telefon Numarası")
    cep_telefonu = fields.String("Cep Telefonu")
    e_posta = fields.String("Kurum E-Posta(Yeni Personel için boş bırakınız)", required=False)
    e_posta_2 = fields.String("E-Posta 2")
    e_posta_3 = fields.String("E-Posta 3")
    web_sitesi = fields.String("Web Sitesi")
    notlar = fields.Text("Not")
    engelli_durumu = fields.String("Engellilik")
    engel_grubu = fields.String("Engel Grubu")
    engel_derecesi = fields.String("Engel Derecesi")
    engel_orani = fields.Integer("Engellilik Orani")



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Birleştirme Sorgula

Hitap üzerinden personelin hizmet birleştirme bilgilerinin sorgulamasını yapar.

"""

from ulakbus.services.personel.hitap.hitap_sorgula import HITAPSorgula


class HizmetBirlestirmeGetir(HITAPSorgula):
    """
    HITAP Sorgulama servisinden kalıtılmış
    Hizmet Birleştirme Bilgisi Sorgulama servisi

    """

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            bean_name (str): Hitap'tan gelen bean nesnesinin adı
            service_dict (dict): Hitap servisinden gelen kayıtların alanları,
                    ``HizmetBirlestirme`` modelinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları listede tutulmaktadır.

        """

        self.service_name = 'HizmetBirlestirmeSorgula'
        self.bean_name = 'HizmetBirlestirmeServisBean'
        self.service_dict = {
            'fields': {
                'tckn': 'tckn',
                'kayit_no': 'kayitNo',
                'sgk_nevi': 'sgkNevi',
                'sgk_sicil_no': 'sgkSicilNo',
                'baslama_tarihi': 'baslamaTarihi',
                'bitis_tarihi': 'bitisTarihi',
                'sure': 'sure',
                'kamu_isyeri_ad': 'kamuIsyeriAd',
                'ozel_isyeri_ad': 'ozelIsyeriAd',
                'bag_kur_meslek': 'bagKurMeslek',
                'ulke_kod': 'ulkeKod',
                'banka_sandik_kod': 'bankaSandikKod',
                'kidem_tazminat_odeme_durumu': 'kidemTazminatOdemeDurumu',
                'ayrilma_nedeni': 'ayrilmaNedeni',
                'kha_durum': 'khaDurum',
                'kurum_onay_tarihi': 'kurumOnayTarihi'
            },
            'date_filter': ['baslama_tarihi', 'bitis_tarihi', 'kurum_onay_tarihi'],
            'required_fields': ['tckn']
        }
        super(HizmetBirlestirmeGetir, self).handle()

    def custom_filter(self, hitap_dict):
        """
        Hitap sözlüğüne uygulanacak ek filtreleri gerçekleştirir.

        Args:
            hitap_dict (List[dict]): Hitap verisini yerele uygun biçimde tutan sözlük listesi

        """

        for record in hitap_dict:
            record['kidem_tazminat_odeme_durumu'] = \
                self.kidem_durum_kontrol(record['kidem_tazminat_odeme_durumu'])
            record['kha_durum'] = self.kha_durum_kontrol(record['kha_durum'])

    def kidem_durum_kontrol(self, kidem_durum):
        """
        Hitap Hizmet Birleştirme servisinin,
        "0" veya "1" olarak gelen Kıdem Tazminatı Ödeme Durumu değeri,
        tam sayı olarak elde edilmektedir.

        - "0": "HAYIR"
        - "1": "EVET"
        - "": "BELİRLENEMEDİ"

        Args:
            kidem_durum (str): Hizmet Birleştirme Kıdem Tazminatı Ödeme Durumu değeri.

        Returns:
            int: Kıdem Tazminatı Ödeme Durumu tam sayı değeri.

        Raises:
            ValueError: Geçersiz Kıdem Tazminatı Ödeme Durumu kodu.
                Varsayılan olarak 2 değeri verilmektedir.

        """

        try:
            return int(kidem_durum)
        except ValueError:
            return 2

    def kha_durum_kontrol(self, kha_durum):
        """
        Hitap Hizmet Birleştirme servisinin,
        "0", "1", "2", "3", "4", "5" olarak gelen Kazanılmış Hak Aylığı
        durum bilgisi değerleri, tam sayı olarak elde edilmektedir.

        - "0": "Değerlendirilmedi"
        - "1": "Prim gün sayısının 2/3 oranında değerlendirildi"
        - "2": "Prim gün sayısının 3/4 oranında değerlendirildi"
        - "3": "Prim gün sayısının 4/4 oranında değerlendirildi"
        - "4": "Belirlenemedi"
        - "5": "İki tarih arasının tamamı değerlendirildi"

        Args:
            kha_durum (str): Hizmet Birleştirme KHA durum değeri.

        Returns:
            int: KHA durum tam sayı değeri.

        Raises:
            ValueError: Geçersiz KHA durum kodu.

        """

        try:
            return int(kha_durum)
        except ValueError:
            self.logger.exception("KHA Durum kodu gecersiz: %s" % kha_durum)
            return 0



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Okul Guncelle

Hitap'a personelin Okul bilgilerinin guncellemesini yapar.

"""

__author__ = 'H.İbrahim Yılmaz (drlinux)'

from ulakbus.services.personel.hitap.hitap_guncelle import HITAPGuncelle


class HizmetOkulGuncelle(HITAPGuncelle):
    """
    HITAP Guncelleme servisinden kalıtılmış Hizmet Okul Bilgi Guncelleme servisi

    """

    def handle(self):
        """Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            service_dict (dict): Request yoluyla gelen kayıtlar,
                    HizmetOkulUpdate servisinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları ve servis tarafında gerekli olan
                    alanlar listede tutulmaktadır.

        """

        self.service_name = 'HizmetOkulUpdate'
        self.service_dict = {
            'fields': {
                'kayitNo': self.request.payload.get('kayit_no', ''),
                'bolum': self.request.payload.get('bolum', ''),
                'kayitNo': self.request.payload.get('kayit_no', ''),
                'mezuniyetTarihi': self.request.payload.get('mezuniyet_tarihi', ''),
                'ogrenimDurumu': self.request.payload.get('ogrenim_durumu', ''),
                'ogrenimSuresi': self.request.payload.get('ogrenim_suresi', ''),
                'okulAd': self.request.payload.get('okul_ad', ''),
                'tckn': self.request.payload.get('tckn', ''),
                'denklikTarihi': self.request.payload.get('denklik_tarihi', ''),
                'ogrenimYer': self.request.payload.get('ogrenim_yeri', ''),
                'denklikBolum': self.request.payload.get('denklik_bolum', ''),
                'denklikOkul': self.request.payload.get('denklik_okul', ''),
                'hazirlik': self.request.payload.get('hazirlik', ''),
                'kurumOnayTarihi': self.request.payload.get('kurum_onay_tarihi', '')
            },
            'date_filter': ['mezuniyetTarihi', 'denklikTarihi', 'kurumOnayTarihi'],
            'required_fields': ['kayitNo', 'tckn', 'ogrenimDurumu', 'mezuniyetTarihi',
                                'ogrenimSuresi', 'hazirlik', 'kurumOnayTarihi']
        }
        super(HizmetOkulGuncelle, self).handle()



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#
# Yeni Personel Ekle WF adimlarini icerir.
from datetime import date

from ulakbus.lib.view_helpers import prepare_choices_for_model
from ulakbus.models.hitap.hitap_sebep import HitapSebep
from ulakbus.models.hitap.hitap import HizmetKayitlari
from zengine.views.crud import CrudView
from zengine.forms import JsonForm, fields
from ulakbus.models.personel import Personel, Atama, Kadro


class PersonelAtama(CrudView):
    class Meta:
        model = 'Personel'

    def eksik_bilgileri_kontrol_et(self):

        try:
            personel = Personel.objects.get(self.input['id'])
        except KeyError:
            personel = Personel.objects.get(self.current.task_data['personel_id'])

        self.current.task_data['personel_id'] = personel.key
        self.current.task_data['personel_ad'] = personel.ad
        self.current.task_data['personel_soyad'] = personel.soyad
        if personel.kurum_sicil_no_int:
            self.current.task_data['eksik_bilgi_yok'] = True
        else:
            self.current.task_data['eksik_bilgi_yok'] = False
            self.current.task_data['ilk_atama'] = True

    def eksik_bilgi_form(self):
        _form = EksikBilgiForm(current=self.current)
        if 'hata_msg' in self.current.task_data:
            _form.help_text = self.current.task_data['hata_msg']

        son_personel = Personel.objects.set_params(sort='kurum_sicil_no_int desc')[0]
        personel = Personel.objects.get(self.current.task_data['personel_id'])

        _form.kurum_sicil_no_int = fields.Integer("Kurum Sicil No (Sıradaki Sicil No : KON-%s)" %
                                                  str(son_personel.kurum_sicil_no_int + 1),
                                                  required=True,
                                                  default=son_personel.kurum_sicil_no_int + 1)

        _form.personel_turu = fields.Integer("Personel Tipi", choices="personel_turu",
                                            default=personel.personel_turu)
        _form.unvan = fields.Integer("Personel Unvan", choices="unvan_kod", required=False,
                                     default=personel.unvan)
        _form.emekli_sicil_no = fields.String("Emekli Sicil No", default=personel.emekli_sicil_no)
        _form.hizmet_sinif = fields.Integer("Hizmet Sınıfı", choices="hizmet_sinifi",
                                            default=personel.hizmet_sinifi)
        _form.statu = fields.Integer("Statü", choices="personel_statu", default=personel.statu)
        _form.brans = fields.String("Branş", default=personel.brans)
        _form.gorev_suresi_baslama = fields.Date("Görev Süresi Başlama",
                                                 default=str(personel.gorev_suresi_baslama))
        _form.gorev_suresi_bitis = fields.Date("Görev Süresi Bitiş",
                                               default=str(personel.gorev_suresi_bitis))
        _form.goreve_baslama_tarihi = fields.Date("Göreve Başlama Tarihi",
                                                  default=str(personel.goreve_baslama_tarihi))

        _form.baslama_sebep = fields.String("Durum", type='typeahead')
        _form.baslama_sebep.default = str(personel.baslama_sebep)
        # TODO: Set choices key error fırlatıyor. düzeltilecek
        # _form.set_choices_of('baslama_sebep', choices=prepare_choices_for_model(HitapSebep, nevi=1))

        _form.mecburi_hizmet_suresi = fields.Date("Mecburi Hizmet Süresi",
                                                  default=str(personel.mecburi_hizmet_suresi))
        _form.emekli_giris_tarihi = fields.Date("Emekliliğe Giriş Tarihi",
                                                default=str(personel.emekli_giris_tarihi))

        self.form_out(_form)

    def eksik_bilgi_kaydet(self):
        personel = Personel.objects.get(self.current.task_data['personel_id'])
        # personel.kurum_sicil_no_int = self.current.input['form']['kurum_sicil_no_int']
        for key in self.current.input['form']:
            setattr(personel, key, self.current.input['form'][key])

        personel.save()

    def atama_durumunu_kontrol_et(self):
        personel = Personel.objects.get(self.current.task_data['personel_id'])
        self.current.task_data['personel'] = personel.clean_value()
        self.current.task_data['guncel_atama'] = personel.atama.key

        if personel.atama.key:
            self.current.task_data['ilk_atama'] = False
        else:
            self.current.task_data['ilk_atama'] = True

    def kadro_bilgileri_form(self):
        _form = KadroBilgiForm(current=self.current,
                               title="%s %s için Atama Yapılacak Kadroyu Seçin" %
                                     (self.current.task_data['personel_ad'],
                                      self.current.task_data['personel_soyad']))

        _form.set_choices_of('kadro', choices=prepare_choices_for_model(Kadro, durum=2))
        _form.set_choices_of('durum', choices=prepare_choices_for_model(HitapSebep, nevi=1))

        self.form_out(_form)

    def kadro_bilgileri_goster(self):
        genel_bilgiler = """**Adı**: {ad}
                           **Soyad**: {soyad}
                           **Personel Tipi**: {personel_turu}""".format(
            **self.current.task_data['personel'])
        atama = Atama.objects.get(self.current.task_data['guncel_atama'])
        atama_data = atama.clean_value()

        atama_bilgileri = "**Hizmet Sınıfı**:\n" \
                          "**Birim**: \n".format(**atama_data)

        output = [{'Genel Bilgiler': genel_bilgiler,
                   'Atama Bilgileri': atama_bilgileri}]

        self.current.output['object'] = {
            "type": "table-multiRow",
            "title": "Personel Atama Bilgileri",
            "fields": output,
            "actions": False
        }

        _form = JsonForm(current=self.current)
        _form.edit = fields.Button("Düzenle", cmd="edit")
        _form.yeni_atama = fields.Button("Atama Yap", cmd="yeni_atama", flow="yeni_atama",
                                         form_validation=False)
        self.form_out(_form)

    def atama_kaydet(self):
        atanacak_kadro = Kadro.objects.get(self.current.input['form']['kadro'])
        if not atanacak_kadro:
            self.current.task_data['hata'] = True
            self.current.task_data['hata_msg'] = "Kadro Dolu olduğu için atama yapılamaz."
        elif atanacak_kadro.durum != 2:
            self.current.task_data['hata'] = True
            self.current.task_data['hata_msg'] = "Kadro Dolu olduğu için atama yapılamaz."
        else:
            self.current.task_data['hata'] = False
            self.current.task_data['kadro'] = str(atanacak_kadro)

            atama = Atama(personel_id=self.current.task_data['personel_id'])
            try:
                atama.kadro = atanacak_kadro
                atama.ibraz_tarihi = self.current.input['form']['ibraz_tarihi']
                atama.durum_id = self.current.input['form']['durum']
                atama.nereden = self.current.input['form']['nereden']
                atama.atama_aciklama = self.current.input['form']['atama_aciklama']
                atama.goreve_baslama_tarihi = self.current.input['form']['goreve_baslama_tarihi']
                atama.goreve_baslama_aciklama = self.current.input['form'][
                    'goreve_baslama_aciklama']
                atama.save()

                personel = Personel.objects.get(self.current.task_data['personel_id'])

                hk = HizmetKayitlari(personel=personel)
                hk.baslama_tarihi = date.today()

                # TODO: Hizmet Kayitlari Model post_save düzgün çalışmadığı için eklendi. #5277
                # Düzeltildiğinde kaldırılacak
                hk.tckn = personel.tckn
                hk.hizmet_sinifi = personel.hizmet_sinifi
                hk.kadro_derece = atanacak_kadro.derece
                hk.odeme_derece = personel.gorev_ayligi_derece
                hk.odeme_kademe = personel.gorev_ayligi_kademe
                hk.odeme_ekgosterge = personel.gorev_ayligi_ekgosterge
                hk.kazanilmis_hak_ayligi_derece = personel.kazanilmis_hak_derece
                hk.kazanilmis_hak_ayligi_kademe = personel.kazanilmis_hak_kademe
                hk.kazanilmis_hak_ayligi_ekgosterge = personel.kazanilmis_hak_ekgosterge
                hk.emekli_derece = personel.emekli_muktesebat_derece
                hk.emekli_kademe = personel.emekli_muktesebat_kademe
                hk.emekli_ekgosterge = personel.emekli_muktesebat_ekgosterge

                hk.sebep_kod = atama.durum.sebep_no
                hk.kurum_onay_tarihi = self.current.input['form']['kurum_onay_tarihi']
                hk.sync = 1  # TODO: Düzeltilecek, beta boyunca senkronize etmemesi için 1 yapıldı
                hk.personel = personel
                hk.save()
            except:
                # Herhangi bir hata oluşursa atama silinecek
                atama.delete(True)

    def atama_goster(self):
        kisi_bilgileri = """**Adı**: {ad}
                           **Soyad**: {soyad}""".format(**self.current.task_data['personel'])

        atama_bilgileri = "**kadro**: {kadro}\n" \
                          "**İlçe**: \n".format(**self.current.task_data)

        output = [{'Kişi Bilgileri': kisi_bilgileri,
                   'Atama Bilgileri': atama_bilgileri}]

        self.current.output['object'] = {
            "type": "table-multiRow",
            "title": "Personel Ataması Başarı ile Tamamlandı",
            "fields": output,
            "actions": False
        }

        _form = JsonForm(current=self.current)

        _form.hitap = fields.Button("Hitap ile Eşleştir", cmd="hitap_getir", btn='hitap')
        _form.bitir = fields.Button("İşlemi Bitir", cmd="bitir", flow="bitir",
                                    form_validation=False)
        self.form_out(_form)

    def atama_iptal(self):
        self.current.output['msgbox'] = {
            'type': 'error', "title": 'Atama İptal Edildi',
            "msg": 'Personel atama işlemi İptal edildi.'
        }

    def sonuc_bilgisi_goster(self):
        hitap_sonuc = ''
        if 'hitap_tamam' in self.current.task_data:
            if self.current.task_data['hitap_tamam']:
                hitap_sonuc = 'Personel için hitap bilgileri Hitap sunucusu ile eşleştirildi.'
            else:
                hitap_sonuc = """
                    Personel için hitap bilgileri Hitap sunucusu ile eşleştirilemedi!!
                    Bu işlemi daha sonra tekrar başlatabilirsiniz.
                    """

        self.current.output['msgbox'] = {
            'type': 'info', "title": 'Personel Atama Başarılı',
            "msg": 'Atama İşlemi Başarıyla gerçekleştirildi. ' + hitap_sonuc
        }

    def hitap_bilgi_getir(self):
        personel = Personel.objects.get(self.current.task_data['personel_id'])
        from ulakbus.services.zato_wrapper import HitapHizmetCetveliSenkronizeEt
        hizmet_cetveli = HitapHizmetCetveliSenkronizeEt(tckn=str(personel.tckn))

        try:
            response = hizmet_cetveli.zato_request()
            self.current.task_data['hitap_tamam'] = True
        except:
            self.current.task_data['hitap_tamam'] = False

        self.current.set_message(
            title='%s TC no için Hitap Hizmet cetveli eşitlendi' % personel.tckn, msg='', typ=1)

    def hatayi_gozden_gecir(self):
        if self.current.task_data['hata_msg']:
            msg = self.current.task_data['hata_msg']
        else:
            msg = "Bilinmeyen bir hata oluştu :( sebebini biliyorsanız bizede söyleyinde düzeltelim"
        self.current.output['msgbox'] = {
            'type': 'error', "title": 'İşlem Başarısız',
            "msg": msg
        }

        _form = JsonForm(current=self.current)
        _form.tekrar = fields.Button("Tekrar Dene", style="btn-success", cmd="tekrar")
        _form.iptal = fields.Button("İptal", cmd="iptal")
        self.form_out(_form)


class EksikBilgiForm(JsonForm):
    class Meta:
        title = 'Eksik Personel Bilgileri'
        help_text = "Atama Öncesi Personelin Eksik Bilgilerini Düzenle."

        grouping = [
            {
                "layout": "6",
                "groups": [
                    {
                        "group_title": "Genel Bilgiler",
                        "items": ['kurum_sicil_no_int', 'personel_turu', 'unvan',
                                  'hizmet_sinif', 'statu', 'brans', 'emekli_sicil_no',
                                  'emekli_giris_tarihi'],
                        "collapse": True,
                    }
                ]
            },
            {
                "layout": "6",
                "groups": [
                    {
                        "group_title": "Çalıştığı birimde işe başlama",
                        "items": ['mecburi_hizmet_suresi'],
                        "collapse": True,
                    }
                ]
            },
            {
                "layout": "6",
                "groups": [
                    {
                        "group_title": "Görev Süresi",
                        "items": ['gorev_suresi_baslama', 'gorev_suresi_bitis',
                                  'engel_derecesi', 'engel_orani', 'baslama_sebep',
                                  'goreve_baslama_tarihi']
                    }
                ]
            }
        ]

    baslama_sebep = None

    kaydet = fields.Button("Kaydet", cmd="kaydet", style="btn-success")
    iptal = fields.Button("İptal", cmd="iptal", flow="iptal", form_validation=False)


class KadroBilgiForm(JsonForm):
    class Meta:
        title = 'Atama Bilgileri'
        help_text = "Yeni Personelin Iletisim Bilgilerini Duzenle."

    kadro = fields.String("Atanacak Kadro Seçiniz", type='typeahead')
    ibraz_tarihi = fields.Date("İbraz Tarihi")
    durum = fields.String("Durum", type='typeahead')
    nereden = fields.Integer("Nereden")
    atama_aciklama = fields.Text("Atama Açıklama")
    goreve_baslama_tarihi = fields.Date("Birimde Göreve Başlama Tarihi")
    goreve_baslama_aciklama = fields.String("Birimde Göreve Başlama Açıklama")
    kurum_onay_tarihi = fields.Date("Kurum Onay Tarihi")

    kaydet = fields.Button("Kaydet", cmd="kaydet", style="btn-success")
    iptal = fields.Button("İptal", cmd="iptal", flow="iptal", form_validation=False)



# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#
"""Danışman Modülü

İlgili İş Akışlarına ait sınıf ve metotları içeren modüldür.

Dönem bazlı danışman atanmasını sağlayan iş akışını yönetir.

"""

from pyoko import ListNode
from zengine import forms
from zengine.forms import fields
from zengine.views.crud import CrudView
from ulakbus.models.ogrenci import Donem, DonemDanisman, Okutman
from ulakbus.models.auth import Unit
from collections import OrderedDict
from ulakbus.views.ders.ders import prepare_choices_for_model


class DonemDanismanForm(forms.JsonForm):
    """``DonemDanismanAtama`` sınıfı için form olarak kullanılacaktır.

    """

    ileri = fields.Button("İleri")


class DonemDanismanListForm(forms.JsonForm):
    class Meta:
        inline_edit = ['secim']

    class Okutmanlar(ListNode):
        secim = fields.Boolean(type="checkbox")
        ad_soyad = fields.String('Ad Soyad')
        key = fields.String('Key', hidden=True)

    kaydet = fields.Button("Kaydet")


class DonemDanismanAtama(CrudView):
    """Dönem Danışman Atama İş Akışı

    Dönem Danışman Atama, aşağıda tanımlı iş akışı adımlarını yürütür.

    - Bölüm Seç
    - Öğretim Elemanlarını Seç
    - Kaydet
    - Kayıt Bilgisi Göster

     Bu iş akışında kullanılan metotlar şu şekildedir:

     Dönem Formunu Listele:
        Kayıtlı dönemleri listeler

     Bölüm Seç:
        Kullanıcının bölüm başkanı olduğu bölümleri listeler

     Öğretim Elemanlarını Seç:
        Seçilen bölümdeki öğretim elemanları listelenir.

     Kaydet:
        Seçilen dönem ve bölüm için seçilen danışman kayıtlarını yapar.

     Kayıt Bilgisi Göster:
        Seçilen öğretim elemanları, dönem ve bölüm bilgilerinden oluşturulan kaydın özeti
        gösterilir.
        Bu adımdan sonra iş akışı sona erer.

     Bu sınıf ``CrudView`` extend edilerek hazırlanmıştır. Temel model
     ``DonemDanisman`` modelidir. Meta.model bu amaçla kullanılmıştır.

     Adımlar arası geçiş manuel yürütülmektedir.

    """

    class Meta:
        model = "DonemDanisman"

    def bolum_sec(self):

        _unit = self.current.role.unit
        _form = DonemDanismanForm(current=self, title="Bölüm Seçiniz")
        _choices = prepare_choices_for_model(Unit, yoksis_no=_unit.yoksis_no)
        _form.program = fields.Integer(choices=_choices)
        self.form_out(_form)

    def danisman_sec(self):

        unit = Unit.objects.get(self.current.input['form']['program'])
        self.current.task_data['unit_yoksis_no'] = unit.yoksis_no
        okutmanlar = Okutman.objects.filter(birim_no=unit.yoksis_no)
        donem = Donem.guncel_donem()
        _form = DonemDanismanListForm(current=self, title="Okutman Seçiniz")

        for okt in okutmanlar:
            try:
                if DonemDanisman.objects.filter(donem=donem, okutman=okt, bolum=unit):
                    _form.Okutmanlar(secim=True, ad_soyad='%s %s' % (okt.ad, okt.soyad),
                                     key=okt.key)
                else:
                    _form.Okutmanlar(secim=False, ad_soyad='%s %s' % (okt.ad, okt.soyad),
                                     key=okt.key)
            except:
                pass

        self.form_out(_form)
        self.current.output["meta"]["allow_actions"] = False
        self.current.output["meta"]["allow_selection"] = False

    def danisman_kaydet(self):
        yoksis_no = self.current.task_data['unit_yoksis_no']
        unit = Unit.objects.get(yoksis_no=yoksis_no)
        donem = Donem.objects.get(guncel=True)
        danismanlar = self.current.input['form']['Okutmanlar']

        # Bölümün ilgili dönemine ait bütün danışmanları sil
        try:
            for dd in DonemDanisman.objects.filter(donem=donem, bolum=unit):
                dd.delete()
        except:
            pass

        for danisman in danismanlar:
            if danisman['secim']:
                key = danisman['key']
                okutman = Okutman.objects.get(key)
                donem_danisman = DonemDanisman()
                donem_danisman.donem = donem
                donem_danisman.okutman = okutman
                donem_danisman.bolum = unit
                try:
                    donem_danisman.save()
                except Exception as e:
                    print(e.message)

    def kayit_bilgisi_ver(self):
        yoksis_no = self.current.task_data['unit_yoksis_no']
        unit = Unit.objects.get(yoksis_no=yoksis_no)
        donem = Donem.objects.get(guncel=True)

        self.current.output['msgbox'] = {
            'type': 'info', "title": 'Danismanlar Kaydedildi',
            "msg": '%s dönemi için %s programına ait danışman listesi kaydedilmiştir' % (
                donem, unit)}



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Birleştirme Ekle

Hitap'a personelin açık süre bilgilerinin eklenmesini yapar.

"""

__author__ = 'H.İbrahim Yılmaz (drlinux)'

from ulakbus.services.personel.hitap.hitap_ekle import HITAPEkle


class HizmetBirlestirmeEkle(HITAPEkle):
    """
    HITAP Ekleme servisinden kalıtılmış Hizmet Birlestirme Bilgisi Ekleme servisi

    """

    def handle(self):
        """Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            service_dict (dict): Request yoluyla gelen kayıtlar,
                    HizmetBirlestirmeInsert servisinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları listede tutulmaktadır.

        """
        self.service_name = 'HizmetBirlestirmeInsert'

        self.service_dict = {
            'fields': {
                'tckn': self.request.payload.get('tckn', ''),
                'sgkNevi': self.request.payload.get('sgk_nevi', ''),
                'sgkSicilNo': self.request.payload.get('sgk_sicil_no', ''),
                'baslamaTarihi': self.request.payload.get('baslama_tarihi', ''),
                'bitisTarihi': self.request.payload.get('bitis_tarihi', ''),
                'kamuIsyeriAd': self.request.payload.get('kamu_isyeri_ad', ''),
                'ozelIsyeriAd': self.request.payload.get('ozel_isyeri_ad', ''),
                'bagKurMeslek': self.request.payload.get('bag_kur_meslek', ''),
                'ulkeKod': self.request.payload.get('ulke_kod', ''),
                'bankaSandikKod': self.request.payload.get('banka_sandik_kod', ''),
                'kidemTazminatOdemeDurumu': self.request.payload.get('kidem_tazminat_odeme_durumu',
                                                                     ''),
                'ayrilmaNedeni': self.request.payload.get('ayrilma_nedeni', ''),
                'sure': self.request.payload.get('sure', ''),
                'khaDurum': self.request.payload.get('kha_durum', ''),
                'kurumOnayTarihi': self.request.payload.get('kurum_onay_tarihi', '')
            },
            'date_filter': ['baslamaTarihi', 'bitisTarihi', 'kurumOnayTarihi'],
            'required_fields': ['tckn', 'sgkNevi', 'sgkSicilNo', 'baslamaTarihi',
                                'bitisTarihi', 'sure', 'kurumOnayTarihi']
        }
        super(HizmetBirlestirmeEkle, self).handle()



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

__author__ = 'H.İbrahim Yılmaz (drlinux)'


class HitapHelper():
    """Hitap Helper
    Zato bazlı HITAP servisleri arasında paylaşımlı olarak kullanılan metodları barındıran classtır.

    """

    def check_required_data(self, hitap_dict):
        """Gelen ``hitap_dict` içindeki ``required_fields`` sözlük listesi içinde belirtilen servis
        tarafında servis tarafında gerekli olarak tanımlanmış alanların hem ``fields`` sözlüğü
        içinde tanımlı olup olmadığını hem de bu alanların değerinin null olmadığını kontrol
        eder.

        Args:
            hitap_dict (dict) : HITAP servisine gönderilmek üzere hazırlanmış sözlük listesi.

        """
        if hitap_dict['required_fields']:
            for required_field in hitap_dict['required_fields']:
                try:
                    if not hitap_dict['fields'][required_field]:
                        raise ValueError("required %s field's value is null" % (required_field))
                except KeyError:
                    raise KeyError("required field %s not found in hitap service dict" % (
                        required_field))



# -*-  coding: utf-8 -*-
"""Roller ile ilgili yardımcı class ve fonksiyonların yer aldığı dosyadır.

"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

__author__ = "H.İbrahim Yılmaz (drlinux)"


class RoleHelper():
    """RoleHelper Class

    """

    def get_abstract_role_name(self, role_key):
        """Verilen role keyine uygun değeri döndürür.

        :param role_key:
        :return: string

        """

        role_dict = {"yukselokul-kurulu-baskani": "Yükselokul Kurulu Başkanı",
                     "enstitu-muduru": "Enstitü Müdürü",
                     "yukselokul-yonetim-kurulu-baskani": "Yükselokul Yönetim Kurulu Başkanı",
                     "fakulte-yonetim-kurulu-uyesi": "Fakülte Yönetim Kurulu Üyesi",
                     "lisans-ogrencisi-kayit-silinmis": "Lisans Programı Öğrencisi - Kayıt Silinmiş",
                     "fakulte-yonetim-kurulu-baskani-dekan": "Fakülte Yönetim Kurulu Başkanı (Dekan)",
                     "fakulte-dekan-sekreteri": "Fakülte Dekan Sekreteri",
                     "fakulte-dekani": "Fakülte Dekanı",
                     "enstitu-ogrenci-isleri-sefi": "Enstitü Öğrenci İşleri Şefi",
                     "enstitu-mudur-yardimcisi": "Enstitü Müdür Yardımcısı",
                     "fakulte-ogrenci-isleri-sefi": "Fakülte Öğrenci İşleri Şefi",
                     "fakulte-kurulu-uyesi": "Fakülte Kurulu Üyesi",
                     "enstitu-muhasebe-isleri-personeli": "Enstitü Muhasebe İşleri Personeli",
                     "on-lisans-ogrencisi-kayit-silinmis": "Ön Lisans Programı Öğrencisi - Kayıt Silinmiş",
                     "yukselokul-birim-koordinatoru": "Yükselokul Birim Koordinatörü",
                     "yukselokul-yonetim-kurulu-uyesi": "Yükselokul Yönetim Kurulu Üyesi",
                     "tip-fakultesi-egitim-komisyonu-uyesi": "Tıp Fakültesi Eğitim Komisyonu Üyesi",
                     "yukselokul-muduru": "Yükselokul Müdürü",
                     "ana-bilim-dali-baskani": "Ana Bilim Dalı Başkanı",
                     "fakulte-sekreteri": "Fakülte Sekreteri",
                     "bolum-kurulu-uyesi": "Bölüm Kurulu Üyesi",
                     "tip-fakultesi-bas-koordinator-yardimcisi": "Tıp Fakültesi Baş Koordinatör Yardımcısı",
                     "fakulte-etik-kurulu-baskani": "Fakülte Etik Kurulu Başkanı",
                     "on-lisans-ogrencisi-kayit-dondurmus": "Ön Lisans Programı Öğrencisi - Kayıt Dondurmuş",
                     "yukselokul-sekreteri": "Yükselokul Sekreteri",
                     "yuksek-lisans-ogrencisi-aktif": "Yüksek Lisans Programı Öğrencisi - Aktif",
                     "fakulte-ogrenci-isleri-personeli": "Fakülte Öğrenci İşleri Personeli",
                     "yukselokul-muhasebe-isleri-sefi": "Yükselokul Muhasebe İşleri Şefi",
                     "ogretim-elemani": "Öğretim Elemanı",
                     "bolum-kurulu-baskani": "Bölüm Kurulu Başkanı",
                     "yuksek-lisans-ogrencisi-kayit-silinmis": "Yüksek Lisans Programı Öğrencisi - Kayıt Silinmiş",
                     "doktora-ogrencisi-kayit-dondurmus": "Doktora Programı Öğrencisi - Kayıt Dondurmuş",
                     "yukselokul-kurulu-uyesi": "Yükselokul Kurulu Üyesi",
                     "fakulte-etik-kurulu-uyesi": "Fakülte Etik Kurulu Üyesi",
                     "lisans-ogrencisi-aktif": "Lisans Programı Öğrencisi - Aktif",
                     "yuksek-lisans-ogrencisi-kayit-dondurmus": "Yüksek Lisans Programı Öğrencisi - Kayıt Dondurmuş",
                     "yukselokul-ogrenci-isleri-sefi": "Yükselokul Öğrenci İşleri Şefi",
                     "enstitu-kurulu-uyesi": "Enstitü Kurulu Üyesi",
                     "bilim-dali-uyesi": "Bilim Dalı Üyesi",
                     "enstitu-muhasebe-isleri-sefi": "Enstitü Muhasebe İşleri Şefi",
                     "lisans-ogrencisi-kayit-dondurmus": "Lisans Programı Öğrencisi - Kayıt Dondurmuş",
                     "enstitu-kurulu-baskani": "Enstitü Kurulu Başkanı",
                     "bolum-sekreteri": "Bölüm Sekreteri",
                     "fakulte-dekan-yardimcisi": "Fakülte Dekan Yardımcısı",
                     "daire-sube-muduru": "Daire Şube Müdürü",
                     "bilim-dali-baskani": "Bilim Dalı Başkanı",
                     "yukselokul-muhasebe-isleri-personeli": "Yükselokul Muhasebe İşleri Personeli",
                     "daire-baskani": "Daire Başkanı",
                     "enstitu-ogrenci-isleri-personeli": "Enstitü Öğrenci İşleri Personeli",
                     "doktora-ogrencisi-aktif": "Doktora Programı Öğrencisi - Aktif",
                     "fakulte-kurulu-baskani-dekan": "Fakülte Kurulu Başkanı (Dekan)",
                     "enstitu-sekreteri": "Enstitü Sekreteri",
                     "tip-fakultesi-egitim-komisyonu-baskani": "Tıp Fakültesi Eğitim Komisyonu Başkanı",
                     "yukselokul-mudur-yardimcisi": "Yükselokul Müdür Yardımcısı",
                     "ana-bilim-dali-uyesi": "Ana Bilim Dalı Üyesi",
                     "daire-personeli": "Daire Personeli",
                     "yukselokul-ogrenci-isleri-personeli": "Yükselokul Öğrenci İşleri Personeli",
                     "enstitu-yonetim-kurulu-baskani": "Enstitü Yönetim Kurulu Başkanı",
                     "tip-fakultesi-bas-koordinatoru": "Tıp Fakültesi Baş Koordinatörü",
                     "bolum-baskani": "Bölüm Başkanı",
                     "enstitu-yonetim-kurulu-uyesi": "Enstitü Yönetim Kurulu Üyesi",
                     "tip-fakultesi-donem-koordinatoru": "Tıp Fakültesi Dönem Koordinatörü",
                     "sube-sefi": "Şube Şefı",
                     "doktora-ogrencisi-kayit-silinmis": "Doktora Programı Öğrencisi - Kayıt Silinmiş",
                     "on-lisans-ogrencisi-aktif": "Ön Lisans Programı Öğrencisi - Aktif"}

        if role_key in role_dict:
            return role_dict[role_key]



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Ünvan Senkronizasyon

Personelin Hitap'taki ünvan bilgilerinin
yereldeki kayıtlarla senkronizasyonunu yapar.

"""

from ulakbus.services.personel.hitap.hitap_sync import HITAPSync
from ulakbus.models.hitap.hitap import HizmetUnvan


class HizmetUnvanSync(HITAPSync):
    """
    HITAP Sync servisinden kalıtılmış Ünvan Bilgisi Senkronizasyon servisi

    """

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Attributes:
            sorgula_service (str): İlgili Hitap sorgu servisinin adı
            model (Model): Hitap'taki kaydın yereldeki karşılığı olan
                        ``HizmetUnvan`` modeli

        """

        self.sorgula_service = 'hizmet-unvan-getir.hizmet-unvan-getir'
        self.model = HizmetUnvan

        super(HizmetUnvanSync, self).handle()



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

# from .personel_test import *



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
from ulakbus.models import AbstractRole
from zengine.management_commands import *


class GenerateFakeData(Command):
    CMD_NAME = 'generate_fake_data'
    HELP = 'Generates fake data for ulakbüs'
    PARAMS = [

        {'name': 'donem_sayisi', 'type': int, 'default': 1,
         'help': 'Üretilecek dönem sayısı, varsayılan 1'},
        {'name': 'kampus_sayisi', 'type': int, 'default': 2,
         'help': 'Üretilecek kampüs sayısı, varsayılan 2'},
        {'name': 'personel_sayisi', 'type': int, 'default': 30,
         'help': 'Üretilecek personel sayısı, varsayılan 30'},
        {'name': 'okutman_sayisi', 'type': int, 'default': 20,
         'help': 'Üretilecek okutman sayısı, varsayılan 20'},
        {'name': 'program_sayisi', 'type': int, 'default': 1,
         'help': 'Üretilecek program sayısı, varsayılan 1'},
        {'name': 'ders_sayisi', 'type': int, 'default': 40,
         'help': 'Üretilecek ders sayısı, varsayılan 40'},
        {'name': 'sube_sayisi', 'type': int, 'default': 3,
         'help': 'Üretilecek şube sayısı, varsayılan 3'},
        {'name': 'sinav_sayisi', 'type': int, 'default': 3,
         'help': 'Üretilecek sınav sayısı, varsayılan 3'},
        {'name': 'ogrenci_sayisi', 'type': int, 'default': 30,
         'help': 'Üretilecek öğrenci sayısı, varsayılan 30'}

    ]

    def run(self):
        from tests.fake.fake_data_generator import FakeDataGenerator
        donem_say = int(self.manager.args.donem_sayisi)
        kampus_say = int(self.manager.args.kampus_sayisi)
        personel_say = int(self.manager.args.personel_sayisi)
        okutman_say = int(self.manager.args.okutman_sayisi)
        program_say = int(self.manager.args.program_sayisi)
        ders_say = int(self.manager.args.ders_sayisi)
        sube_say = int(self.manager.args.sube_sayisi)
        sinav_say = int(self.manager.args.sinav_sayisi)
        ogrenci_say = int(self.manager.args.ogrenci_sayisi)
        fake = FakeDataGenerator()
        fake.fake_data(donem_say=donem_say, kampus_say=kampus_say, personel_say=personel_say,
                       okutman_say=okutman_say,
                       program_say=program_say, ders_say=ders_say, sinav_say=sinav_say,
                       sube_say=sube_say,
                       ogrenci_say=ogrenci_say)


class GenerateAbstractRoles(Command):
    CMD_NAME = 'generate_abstract_roles'
    HELP = 'Generates abstract roles'
    ROLE_LIST = [
        "Lisans Programı Öğrencisi - Aktif",
        "Lisans Programı Öğrencisi - Kayıt Dondurmuş",
        "Lisans Programı Öğrencisi - Kayıt Silinmiş",

        "Ön Lisans Programı Öğrencisi - Aktif",
        "Ön Lisans Programı Öğrencisi - Kayıt Dondurmuş",
        "Ön Lisans Programı Öğrencisi - Kayıt Silinmiş",

        "Yüksek Lisans Programı Öğrencisi - Aktif",
        "Yüksek Lisans Programı Öğrencisi - Kayıt Dondurmuş",
        "Yüksek Lisans Programı Öğrencisi - Kayıt Silinmiş",

        "Doktora Programı Öğrencisi - Aktif",
        "Doktora Programı Öğrencisi - Kayıt Dondurmuş",
        "Doktora Programı Öğrencisi - Kayıt Silinmiş",

        "Fakülte Dekanı",
        "Fakülte Dekan Sekreteri",
        "Fakülte Dekan Yardımcısı",
        "Fakülte Kurulu Başkanı (Dekan)",
        "Fakülte Kurulu Üyesi",
        "Fakülte Yönetim Kurulu Başkanı (Dekan)",
        "Fakülte Yönetim Kurulu Üyesi",
        "Fakülte Sekreteri",
        "Fakülte Etik Kurulu Başkanı",
        "Fakülte Etik Kurulu Üyesi",
        "Fakülte Öğrenci İşleri Şefi",
        "Fakülte Öğrenci İşleri Personeli",

        "Tıp Fakültesi Baş Koordinatörü",
        "Tıp Fakültesi Baş Koordinatör Yardımcısı",
        "Tıp Fakültesi Dönem Koordinatörü",
        "Tıp Fakültesi Eğitim Komisyonu Başkanı",
        "Tıp Fakültesi Eğitim Komisyonu Üyesi",

        "Yükselokul Müdürü",
        "Yükselokul Müdür Yardımcısı",
        "Yükselokul Kurulu Başkanı",
        "Yükselokul Kurulu Üyesi",
        "Yükselokul Yönetim Kurulu Başkanı",
        "Yükselokul Yönetim Kurulu Üyesi",
        "Yükselokul Sekreteri",
        "Yükselokul Öğrenci İşleri Şefi",
        "Yükselokul Öğrenci İşleri Personeli",
        "Yükselokul Muhasebe İşleri Şefi",
        "Yükselokul Muhasebe İşleri Personeli",
        "Yükselokul Birim Koordinatörü",

        "Bölüm Başkanı",
        "Bölüm Kurulu Başkanı",
        "Bölüm Kurulu Üyesi",
        "Bölüm Sekreteri",

        "Bilim Dalı Başkanı",
        "Bilim Dalı Üyesi",

        "Ana Bilim Dalı Başkanı",
        "Ana Bilim Dalı Üyesi",

        "Enstitü Müdürü",
        "Enstitü Müdür Yardımcısı",
        "Enstitü Kurulu Başkanı",
        "Enstitü Kurulu Üyesi",
        "Enstitü Yönetim Kurulu Başkanı",
        "Enstitü Yönetim Kurulu Üyesi",
        "Enstitü Sekreteri",
        "Enstitü Öğrenci İşleri Şefi",
        "Enstitü Öğrenci İşleri Personeli",
        "Enstitü Muhasebe İşleri Şefi",
        "Enstitü Muhasebe İşleri Personeli",

        "Öğretim Elemanı",
        # "Öğretim Üyesi",
        # "Öğretim Görevlisi",
        # "Araştırma Görevlisi",
        # "Okutman",
        # "Uzman",
        # "Çevirici",
        # "Eğitim-öğretim Planlamacısı",

        "Daire Başkanı",
        "Daire Şube Müdürü",
        "Şube Şefı",
        "Daire Personeli",

    ]

    def run(self):
        for role in self.ROLE_LIST:
            AbstractRole(name=role).save()


# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
from collections import defaultdict

from ulakbus.lib.convert import tl_curreny
from ulakbus.models import *
from ulakbus.views.reports.base import Reporter
from zengine.lib.utils import *


class OgrenciByGender(Reporter):
    TITLE = 'Cinsiyete göre öğrenci sayıları'

    def get_objects(self):
        choices = self.convert_choices(Ogrenci().get_choices_for('cinsiyet'))
        result = []
        for val, num in Ogrenci.objects.distinct_values_of('cinsiyet').items():
            try:
                val = int(val)
            except:
                pass
            result.append((choices.get(val, val), num))
        return result


class OgrenciByBrithPlace(Reporter):
    TITLE = 'Doğum yerine göre öğrenci sayıları'

    def get_objects(self):
        return [(val, num) for val, num in
                Ogrenci.objects.distinct_values_of('dogum_yeri').items()]


class OgrenciByBrithDate(Reporter):
    TITLE = 'Doğum tarihine göre öğrenci sayıları'

    def get_objects(self):
        dates = defaultdict(lambda: 0)
        for val, num in Ogrenci.objects.distinct_values_of('dogum_tarihi').items():
            dates[solr_to_year(val)] += int(num)
        return dates.items()


class OgrenciHarc(Reporter):
    TITLE = 'Harç Bilgileri'

    def get_objects(self):
        # choices = self.convert_choices(Borc().get_choices_for('sebep'))
        result = defaultdict(lambda: 0)
        for b in Borc.objects.filter():
            result["%s %s" % (b.get_sebep_display(), 'Borç')] += int(b.miktar or 0)
            result["%s %s" % (b.get_sebep_display(), 'Ödenen')] += int(b.odenen_miktar or 0)

        return [(k, tl_curreny(v)) for k, v in result.items()]


class RoomCapacities(Reporter):
    TITLE = 'Kapasitesine Göre Mekanlar'

    def get_objects(self):
        return [{'Kapasite': k, 'Oda Sayısı': v} for k, v in
                     Room.objects.distinct_values_of('capacity').items()]
        # return Room.objects.distinct_values_of('capacity').items()



# -*- coding:utf-8 -*-

from pyoko import Model, field


class HitapSebep(Model):
    sebep_no = field.Integer("Sebep No")
    ad = field.String("Sebep Adı")
    nevi = field.Integer("Sebep Nevi")
    zorunlu_alan = field.String("Zorunlu ALan")

    class Meta:
        app = 'Personel'
        verbose_name = "Hitap Sebep Kodu"
        verbose_name_plural = "Hitap Sebep Kodları"
        list_fields = ['sebep_no', 'ad', 'nevi', 'zorunlu_alan']
        search_fields = ['sebep_no', 'ad']

    def __unicode__(self):
        return '%s - %s' % (self.sebep_no, self.ad)



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

import os
import sys
from zengine.management_commands import *
from lxml import etree
from ulakbus.models import Donem, Ogrenci, Unit, Sube, Ders, Program, OgrenciProgram, OgrenciDersi, Okutman, Campus
import datetime


class UnitimeEntityXMLExport(Command):
    EXPORT_DIR = 'bin/dphs/data_exchange/'
    FILE_NAME = ''
    DOC_TYPE = ''
    CMD_NAME = ''
    HELP = ''
    PARAMS = []

    def write_file(self, data):
        out_dir = self.create_dir()
        out_file = open(out_dir + '/' + self.FILE_NAME, 'w+')
        out_file.write("%s" % data)
        print(
            "Veriler %s dizini altinda %s adlı dosyaya kayit edilmiştir" % (
                out_dir, self.FILE_NAME))

    def run(self):
        data = self.prepare_data()
        if len(data) > 0:
            self.write_file(data)
        else:
            print("Aktarilacak veri bulunamadi!.")

    def prepare_data(self):
        return ''

    def create_dir(self):
        current_date = datetime.datetime.now()
        export_directory = self.EXPORT_DIR + current_date.strftime('%d_%m_%Y_%H')
        if not os.path.exists(export_directory):
            os.makedirs(export_directory)
        return export_directory

    @property
    def uni(self):
        try:
            return Unit.objects.filter(parent_unit_no=0)[0].yoksis_no
        except:
            print("Universite tanimlanmamis")
            sys.exit(1)

    @property
    def term(self):
        try:
            return Donem.objects.filter(guncel=True)[0]
        except:
            print("Guncel donem tanimlanmamis")
            sys.exit(1)

    @property
    def campuses(self):
        c = Campus.objects.filter()
        if len(c) > 0:
            return c
        else:
            print("Kampus Bulunamadi")
            sys.exit(1)

    @property
    def sessions(self):
        s = Donem.objects.filter()
        if len(s) > 0:
            return s
        else:
            print("Donem Bulunamadi")
            sys.exit(1)

    @property
    def bolumler(self):
        b = Unit.objects.filter(unit_type='Bölüm')
        if len(b) > 0:
            return b
        else:
            print("Bolum Bulunamadi")
            sys.exit(1)


class ExportRooms(UnitimeEntityXMLExport):
    CMD_NAME = 'export_rooms'
    HELP = 'Generates Unitime XML import file for rooms'
    PARAMS = []
    FILE_NAME = 'buildingRoomImport.xml'
    DOC_TYPE = '<!DOCTYPE buildingsRooms PUBLIC "-//UniTime//DTD University Course Timetabling/EN" "http://www.unitime.org/interface/BuildingRoom.dtd">'

    def prepare_data(self):
        # create XML
        root = ''
        for campus in self.campuses:
            if campus.building_set:
                root = etree.Element('buildingsRooms', campus="%s" % self.uni,
                                     term="%s" % self.term.ad,
                                     year="%s" % self.term.baslangic_tarihi.year)
                for building in campus.building_set:

                    buildingelement = etree.SubElement(
                            root, 'building',
                            externalId="%s" % building.building.key,
                            abbreviation="%s" % building.building.code,
                            locationX="%s" % building.building.coordinate_x,
                            locationY="%s" % building.building.coordinate_y,
                            name="%s" % building.building.name)
                    if building.building.room_set:

                        for room in building.building.room_set:
                            roomelement = etree.SubElement(
                                    buildingelement, 'room',
                                    externalId="%s" % room.room.key,
                                    locationX="%s" % building.building.coordinate_x,
                                    locationY="%s" % building.building.coordinate_y,
                                    roomNumber="%s" % room.room.code,
                                    roomClassification="%s" % room.room.room_type.type,
                                    capacity="%s" % room.room.capacity,
                                    instructional="True")

                            if room.room.RoomDepartments:
                                roommdepartments = etree.SubElement(roomelement,
                                                                    'roomDepartments')
                                for department in room.room.RoomDepartments:
                                    etree.SubElement(
                                            roommdepartments, 'assigned',
                                            departmentNumber="%s" % department.unit.yoksis_no,
                                            percent="100")

        # pretty string
        return etree.tostring(root, pretty_print=True, xml_declaration=True, encoding='UTF-8',
                              doctype="%s" % self.DOC_TYPE)


class ExportSessionsToXml(UnitimeEntityXMLExport):
    CMD_NAME = 'export_sessions'
    HELP = 'Generates Unitime XML import file for academic sessions'
    PARAMS = []
    FILE_NAME = 'sessionImport.xml'
    DOC_TYPE = '<!DOCTYPE session PUBLIC "-//UniTime//DTD University Course Timetabling/EN" "http://www.unitime.org/interface/Session.dtd">'

    def prepare_data(self):
        root = ''
        # create XML
        for campus in self.campuses:

            root = etree.Element('session', campus="%s" % self.uni, term="%s" % self.term.ad,
                                 year="%s" % self.term.baslangic_tarihi.year, dateFormat="M/d/y")
            for session in self.sessions:
                start_date = session.baslangic_tarihi.strftime("%m/%d/%Y")
                end_date = session.bitis_tarihi.strftime("%m/%d/%Y")
                etree.SubElement(root, 'sessionDates', beginDate="%s" % start_date,
                                 endDate="%s" % end_date,
                                 classesEnd="%s" % end_date, examBegin="%s" % start_date,
                                 eventBegin="%s" % start_date, eventEnd="%s" % end_date)
        # pretty string
        return etree.tostring(root, pretty_print=True, xml_declaration=True, encoding='UTF-8',
                              doctype="%s" % self.DOC_TYPE)


class ExportDepartmentsToXML(UnitimeEntityXMLExport):
    CMD_NAME = 'export_departments'
    HELP = 'Generates Unitime XML import file for academic departments'
    PARAMS = []
    FILE_NAME = 'departmentImport.xml'
    DOC_TYPE = '<!DOCTYPE departments PUBLIC "-//UniTime//DTD University Course Timetabling/EN" "http://www.unitime.org/interface/Department.dtd">'

    def prepare_data(self):

        # create XML
        for campus in self.campuses:

            root = etree.Element('departments', campus="%s" % self.uni, term="%s" % self.term.ad,
                                 year="%s" % self.term.baslangic_tarihi.year)
            for unit in self.bolumler:
                etree.SubElement(root, 'department', externalId="%s" % unit.key,
                                 abbreviation="%s" % unit.yoksis_no, name="%s" % unit.name,
                                 deptCode="%s" % unit.yoksis_no, allowEvents="true")
        # pretty string
        return etree.tostring(root, pretty_print=True, xml_declaration=True, encoding='UTF-8',
                              doctype="%s" % self.DOC_TYPE)


class ExportAcademicSubjectsToXML(UnitimeEntityXMLExport):
    CMD_NAME = 'export_academic_subjects'
    HELP = 'Generates Unitime XML import file for academic subjects'
    PARAMS = []
    FILE_NAME = 'subjectAreaImport.xml'
    DOC_TYPE = '<!DOCTYPE subjectAreas PUBLIC "-//UniTime//DTD University Course Timetabling/EN" "http://www.unitime.org/interface/SubjectArea.dtd">'

    def prepare_data(self):

        # create XML
        for campus in self.campuses:

            root = etree.Element('subjectAreas', campus="%s" % self.uni, term="%s" % self.term.ad,
                                 year="%s" % self.term.baslangic_tarihi.year)
            for unit in self.bolumler:
                try:
                    subunits = Unit.objects.filter(parent_unit_no=unit.yoksis_no)
                    for subunit in subunits:
                        etree.SubElement(root, 'subjectArea', externalId="%s" % subunit.key,
                                         abbreviation="%s" % subunit.yoksis_no,
                                         title="%s" % subunit.name,
                                         department="%s" % subunit.parent_unit_no)
                except:
                    pass

        # pretty string
        return etree.tostring(root, pretty_print=True, xml_declaration=True, encoding='UTF-8',
                              doctype="%s" % self.DOC_TYPE)


class ExportStaffToXML(UnitimeEntityXMLExport):
    CMD_NAME = 'export_staff'
    HELP = 'Generates Unitime XML import file for staff'
    PARAMS = []
    FILE_NAME = 'staffImport.xml'
    DOC_TYPE = '<!DOCTYPE staff PUBLIC "-//UniTime//UniTime Staff Import DTD/EN" "http://www.unitime.org/interface/Staff.dtd">'

    def prepare_data(self):
        stafflist = Okutman.objects.filter()
        if len(stafflist) > 0:
            for campus in self.campuses:
                root = etree.Element('staff', campus="%s" % self.uni, term="%s" % self.term.ad,
                                     year="%s" % self.term.baslangic_tarihi.year)
                for staffmember in stafflist:
                    unvan = self.acadTitle(title=staffmember.unvan)
                    staff_dep = staffmember.birim_no
                    if staffmember.birim_no:
                        try:
                            staff_dep = Unit.objects.filter(yoksis_no=staffmember.birim_no)[
                                0].parent_unit_no
                            etree.SubElement(root, 'staffMember', externalId="%s" % staffmember.key,
                                             firstName="%s" % staffmember.ad,
                                             lastName="%s" % staffmember.soyad,
                                             department="%s" % staff_dep, acadTitle="%s" % unvan[0],
                                             positionType="%s" % unvan[1])
                        except:
                            pass
            # pretty string
            return etree.tostring(root, pretty_print=True, xml_declaration=True, encoding='UTF-8',
                                  doctype="%s" % self.DOC_TYPE)
        else:
            print("Okutman Bulunamadi")

    @staticmethod
    def acadTitle(title):
        if title == 1:
            return ["Professor", "PROF"]
        elif title == 2:
            return ["Associate Professor", "ASSOC_PROF"]
        elif title == 3:
            return ["Research Assistant", "INSTRUCTOR"]
        elif title == 4:
            return ["Lecturer", "INSTRUCTOR"]
        else:
            return ["", ""]


class ExportStudentInfoToXML(UnitimeEntityXMLExport):
    CMD_NAME = 'export_student_info'
    HELP = 'Generates Unitime XML import file for student info'
    PARAMS = [

        {'name': 'batch_size', 'type': int, 'default': 1000,
         'help': 'Retrieve this amount of records from Solr in one time, defaults to 1000'},

    ]
    FILE_NAME = 'studentInfoImport.xml'
    DOC_TYPE = '<!DOCTYPE students PUBLIC "-//UniTime//DTD University Course Timetabling/EN" "http://www.unitime.org/interface/Student.dtd">'

    def prepare_data(self):
        # FIX for default row size in pyoko filter
        batch_size = int(self.manager.args.batch_size)
        count = Ogrenci.objects.count()
        rounds = int(count / batch_size) + 1

        root = etree.Element('students', campus="%s" % self.uni, term="%s" % self.term.ad,
                             year="%s" % self.term.baslangic_tarihi.year)
        # FIX for default row size in pyoko filter
        for i in range(rounds):
            for student in Ogrenci.objects.set_params(rows=1000, start=i * batch_size).filter():
                etree.SubElement(root, 'student', externalId="%s" % student.key,
                                 firstName="%s" % student.ad,
                                 lastName="%s" % student.soyad,
                                 email="%s" % student.e_posta)
        # pretty string
        return etree.tostring(root, pretty_print=True, xml_declaration=True, encoding='UTF-8',
                              doctype="%s" % self.DOC_TYPE)


class ExportCourseCatalogToXML(UnitimeEntityXMLExport):
    CMD_NAME = 'export_course_catalog'
    HELP = 'Generates Unitime XML import file for course catalog'
    PARAMS = [

        {'name': 'batch_size', 'type': int, 'default': 1000,
         'help': 'Retrieve this amount of records from Solr in one time, defaults to 1000'},

    ]
    FILE_NAME = 'courseCatalogImport.xml'
    DOC_TYPE = '<!DOCTYPE courseCatalog PUBLIC "-//UniTime//DTD University Course Timetabling/EN" "http://www.unitime.org/interface/CourseCatalog.dtd">'

    def prepare_data(self):
        # FIX for default row size in pyoko filter
        batch_size = int(self.manager.args.batch_size)
        count = Ders.objects.count()
        rounds = int(count / batch_size) + 1

        root = etree.Element('courseCatalog', campus="%s" % self.uni, term="%s" % self.term.ad,
                             year="%s" % self.term.baslangic_tarihi.year)
        # FIX for default row size in pyoko filter
        for i in range(rounds):
            for ders in Ders.objects.set_params(rows=1000, start=i * batch_size).filter():
                derselement = etree.SubElement(root, 'course', externalId="%s" % ders.key,
                                               courseNumber="%s" % ders.kod,
                                               subject="%s" % ders.program.yoksis_no,
                                               title="%s" % ders.ad)
                etree.SubElement(derselement, 'courseCredit', creditType="collegiate",
                                 creditUnitType="semesterHours",
                                 creditFormat="fixedUnit",
                                 fixedCredit="%s" % ders.yerel_kredisi)
        # pretty string
        return etree.tostring(root, pretty_print=True, xml_declaration=True, encoding='UTF-8',
                              doctype="%s" % self.DOC_TYPE)


class ExportCurriculaToXML(UnitimeEntityXMLExport):
    CMD_NAME = 'export_curricula'
    HELP = 'Generates Unitime XML import file for curricula'
    PARAMS = []
    FILE_NAME = 'curricula.xml'
    DOC_TYPE = '<!DOCTYPE curricula PUBLIC "-//UniTime//DTD University Course Timetabling/EN" "http://www.unitime.org/interface/Curricula_3_2.dtd">'

    def prepare_data(self):

        program_list = Program.objects.filter()

        if len(program_list) > 0:

            """
            curricula Import File

            """

            root = etree.Element('curricula', campus="%s" % self.uni, term="%s" % self.term.ad,
                                 year="%s" % self.term.baslangic_tarihi.year)
            for program in program_list:
                ders_list = Ders.objects.filter(program=program).count()
                try:
                    parent_yoksis_no = Unit.objects.filter(yoksis_no=program.yoksis_no)[
                        0].parent_unit_no
                    curriculum = etree.SubElement(root, 'curriculum')
                    etree.SubElement(curriculum, 'academicArea', abbreviation='A')
                    etree.SubElement(curriculum, 'department', code="%s" % parent_yoksis_no)
                    etree.SubElement(curriculum, 'major', code="M1")
                    classification = etree.SubElement(curriculum, 'classification', name="01",
                                                      enrollment='2')
                    etree.SubElement(classification, 'academicClassification', code="01")
                    for program_ders in Ders.objects.filter(program=program):
                        etree.SubElement(classification, 'course', subject="%s" % program.yoksis_no,
                                         courseNbr="%s" % program_ders.kod)
                except:
                    pass

            # pretty string
            return etree.tostring(root, pretty_print=True, xml_declaration=True, encoding='UTF-8',
                                  doctype="%s" % self.DOC_TYPE)
        else:
            print("Program Bulunmadi")


class ExportAcademicAreaToXML(UnitimeEntityXMLExport):
    CMD_NAME = 'export_academic_area'
    HELP = 'Generates Unitime XML import file for Academic Areas'
    PARAMS = []
    FILE_NAME = 'academicAreaImport.xml'
    DOC_TYPE = '<!DOCTYPE academicAreas PUBLIC "-//UniTime//DTD University Course Timetabling/EN" "http://www.unitime.org/interface/AcademicArea.dtd">'

    def prepare_data(self):
        """
        academicAreas Import File

        """

        root = etree.Element('academicAreas', campus="%s" % self.uni, term="%s" % self.term.ad,
                             year="%s" % self.term.baslangic_tarihi.year)
        etree.SubElement(root, 'academicArea', abbreviation='A', externalId='A',
                         title="%s" % self.uni + ' - ' + self.term.ad)

        return etree.tostring(root, pretty_print=True, xml_declaration=True, encoding='UTF-8',
                              doctype="%s" % self.DOC_TYPE)


class ExportAcademicClassificationsToXML(UnitimeEntityXMLExport):
    CMD_NAME = 'export_academic_class'
    HELP = 'Generates Unitime XML import file for Academic Classifications'
    PARAMS = []
    FILE_NAME = 'academicClassificationImport.xml'
    DOC_TYPE = '<!DOCTYPE academicClassifications PUBLIC "-//UniTime//DTD University Course Timetabling/EN" "http://www.unitime.org/interface/AcademicClassification.dtd">'

    def prepare_data(self):
        """
        academicClassifications Import File

        """

        root = etree.Element('academicClassifications', campus="%s" % self.uni,
                             term="%s" % self.term.ad,
                             year="%s" % self.term.baslangic_tarihi.year)
        etree.SubElement(root, 'academicClassification',
                         externalId="01",
                         code="01", name="01")
        return etree.tostring(root, pretty_print=True, xml_declaration=True, encoding='UTF-8',
                              doctype="%s" % self.DOC_TYPE)


class ExportPosMajorsToXML(UnitimeEntityXMLExport):
    CMD_NAME = 'export_posmajors'
    HELP = 'Generates Unitime XML import file for Pos Majors'
    PARAMS = []
    FILE_NAME = 'posMajorImport.xml'
    DOC_TYPE = '<!DOCTYPE posMajors PUBLIC "-//UniTime//DTD University Course Timetabling/EN" "http://www.unitime.org/interface/PosMajor.dtd">'

    def prepare_data(self):
        """
        posMajors Import File

        """

        root = etree.Element('posMajors', campus="%s" % self.uni, term="%s" % self.term.ad,
                             year="%s" % self.term.baslangic_tarihi.year)
        etree.SubElement(root, 'posMajor', externalId="M1", code="M1",
                         name="%s Major 1" % self.term.baslangic_tarihi.year,
                         academicArea="A")
        return etree.tostring(root, pretty_print=True, xml_declaration=True, encoding='UTF-8',
                              doctype="%s" % self.DOC_TYPE)


class ExportStudentCourseDemandsToXML(UnitimeEntityXMLExport):
    CMD_NAME = 'export_student_course_demands'
    HELP = 'Generates Unitime XML import file for student course demands'
    PARAMS = [

        {'name': 'batch_size', 'type': int, 'default': 1000,
         'help': 'Retrieve this amount of records from Solr in one time, defaults to 1000'},

    ]
    FILE_NAME = 'studentCrsDemandImport.xml'
    DOC_TYPE = '<!DOCTYPE lastLikeCourseDemand PUBLIC "-//UniTime//DTD University Course Timetabling/EN" "http://www.unitime.org/interface/StudentCourse.dtd">'

    def prepare_data(self):

        program_list = Program.objects.filter()

        if len(program_list) > 0:
            '''
            lastLikeCourseDemand Import File
            '''
            root = etree.Element('lastLikeCourseDemand', campus="%s" % self.uni,
                                 term="%s" % self.term.ad,
                                 year="%s" % self.term.baslangic_tarihi.year)

            batch_size = int(self.manager.args.batch_size)
            count = Ogrenci.objects.count()
            rounds = int(count / batch_size) + 1

            for i in range(rounds):
                for student in Ogrenci.objects.set_params(rows=1000, start=i * batch_size).filter():
                    student_department_list = OgrenciProgram.objects.filter(ogrenci=student)
                    if (student_department_list):
                        student_element = etree.SubElement(root, "student",
                                                           externalId="%s" % student.key)
                        for department in student_department_list:
                            try:
                                for ogrenci_ders in OgrenciDersi.objects.filter(
                                        ogrenci_program=department):
                                    ders = ogrenci_ders.ders
                                    etree.SubElement(student_element, 'studentCourse',
                                                     externalId="%s" % ders.key,
                                                     courseNumber="%s" % ders.kod,
                                                     subject="%s" % ders.program.yoksis_no)
                            except:
                                pass
            # pretty string
            return etree.tostring(root, pretty_print=True, xml_declaration=True, encoding='UTF-8',
                                  doctype="%s" % self.DOC_TYPE)
        else:
            print("Program Bulunamadi")


class ExportStudentCoursesToXML(UnitimeEntityXMLExport):
    CMD_NAME = 'export_student_courses'
    HELP = 'Generates Unitime XML import file for student courses'
    PARAMS = [

        {'name': 'batch_size', 'type': int, 'default': 1000,
         'help': 'Retrieve this amount of records from Solr in one time, defaults to 1000'},

    ]
    FILE_NAME = 'studentEnrollments.xml'
    DOC_TYPE = '<!DOCTYPE studentEnrollments PUBLIC "-//UniTime//DTD University Course Timetabling/EN" "http://www.unitime.org/interface/StudentEnrollment.dtd">'

    def prepare_data(self):
        program_list = Program.objects.filter()

        if len(program_list) > 0:

            '''
            studentEnrollments Import File
            '''

            root = etree.Element('studentEnrollments', campus="%s" % self.uni,
                                 term="%s" % self.term.ad,
                                 year="%s" % self.term.baslangic_tarihi.year)

            batch_size = int(self.manager.args.batch_size)
            count = Ogrenci.objects.count()
            rounds = int(count / batch_size) + 1

            for i in range(rounds):
                for student in Ogrenci.objects.set_params(rows=1000, start=i * batch_size).filter():
                    student_department_list = OgrenciProgram.objects.filter(ogrenci=student)
                    if len(student_department_list) > 0:
                        student_element = etree.SubElement(root, "student",
                                                           externalId="%s" % student.key,
                                                           firstName="%s" % student.ad,
                                                           lastName="%s" % student.soyad,
                                                           email="%s" % student.e_posta)
                        for department in student_department_list:
                            try:
                                for ogrenci_ders in OgrenciDersi.objects.filter(
                                        ogrenci_program=department):
                                    if (ogrenci_ders):
                                        ders = ogrenci_ders.ders
                                        etree.SubElement(student_element, 'class',
                                                         externalId="%s" % ders.key,
                                                         courseNbr="%s" % ders.kod,
                                                         subject="%s" % ders.program.yoksis_no,
                                                         type="Lec", suffix="1")
                            except:
                                pass
            # pretty string
            return etree.tostring(root, pretty_print=True, xml_declaration=True, encoding='UTF-8',
                                  doctype="%s" % self.DOC_TYPE)
        else:
            print("Program Bulunamadi")


class ExportClassesToXML(UnitimeEntityXMLExport):
    CMD_NAME = 'export_classes'
    HELP = 'Generates Unitime XML import file for timetable'
    PARAMS = [

        {'name': 'batch_size', 'type': int, 'default': 1000,
         'help': 'Retrieve this amount of records from Solr in one time, defaults to 1000'},

    ]
    FILE_NAME = 'courseTimetableImport.xml'
    DOC_TYPE = '<!DOCTYPE timetable PUBLIC "-//UniTime//DTD University Course Timetabling/EN" "http://www.unitime.org/interface/CourseTimetable.dtd">'

    def prepare_data(self):

        """
        courseTimetableImport Import File

        """

        batch_size = int(self.manager.args.batch_size)
        count = Sube.objects.filter(donem=self.term).count()
        rounds = int(count / batch_size) + 1
        root = etree.Element('timetable', campus="%s" % self.uni, term="%s" % self.term.ad,
                             year="%s" % self.term.baslangic_tarihi.year, action="insert",
                             instructors="false",
                             notes="false", prefer="id", timeFormat="HHmm",
                             dateFormat="yyyy/M/d")

        for i in range(rounds):
            for sube in Sube.objects.set_params(rows=1000, start=i * batch_size).filter(
                    donem=self.term):
                lecture = sube.ders
                etree.SubElement(root, 'class', name="%s" % sube.ad,
                                 courseNbr="%s" % lecture.kod,
                                 subject="%s" % lecture.program.yoksis_no,
                                 type="Lec")
        # pretty string
        return etree.tostring(root, pretty_print=True, xml_declaration=True, encoding='UTF-8',
                              doctype="%s" % self.DOC_TYPE)


class ExportCorseOfferingsToXML(UnitimeEntityXMLExport):
    CMD_NAME = 'export_course_offerings'
    HELP = 'Generates Unitime XML import file for timetable'
    PARAMS = [

        {'name': 'batch_size', 'type': int, 'default': 1000,
         'help': 'Retrieve this amount of records from Solr in one time, defaults to 1000'},

    ]
    FILE_NAME = 'courseOfferingsImport.xml'
    DOC_TYPE = '<!DOCTYPE offerings PUBLIC "-//UniTime//DTD University Course Timetabling/EN" "http://www.unitime.org/interface/CourseOfferingExport.dtd">'

    def prepare_data(self):

        """
        offerings Import File

        """

        batch_size = int(self.manager.args.batch_size)
        count = Sube.objects.filter(donem=self.term).count()
        rounds = int(count / batch_size) + 1
        root = etree.Element('offerings', campus="%s" % self.uni, term="%s" % self.term.ad,
                             year="%s" % self.term.baslangic_tarihi.year, action="insert",
                             incremental="true",
                             timeFormat="HHmm", dateFormat="yyyy/M/d")

        for i in range(rounds):
            for sube in Sube.objects.set_params(rows=1000, start=i * batch_size).filter(
                    donem=self.term):

                try:
                    lecture = sube.ders
                    offering_elem = etree.SubElement(root, 'offering', id="%s" % sube.ad,
                                                     offered="true")
                    course_elem = etree.SubElement(offering_elem, 'course',
                                                   courseNbr="%s" % lecture.kod,
                                                   subject="%s" % lecture.program.yoksis_no,
                                                   controlling="true")
                    etree.SubElement(course_elem, 'class', name="%s" % sube.ad, suffix="1",
                                     limit="%s" % sube.kontenjan,
                                     type="Rec")
                except:
                    pass

        # pretty string
        return etree.tostring(root, pretty_print=True, xml_declaration=True, encoding='UTF-8',
                              doctype="%s" % self.DOC_TYPE)


class ExportAllUnitimeXMLs(Command):
    CMD_NAME = 'export_all_unitime_xmls'
    HELP = 'Tum unitime xml dosyalarini tek bir dizine aktarir.'

    def run(self):
        ExportSessionsToXml().run()
        ExportRooms().run()
        ExportDepartmentsToXML().run()
        ExportAcademicSubjectsToXML().run()
        ExportStaffToXML().run()
        ExportStudentInfoToXML().run()
        ExportCourseCatalogToXML().run()
        ExportCorseOfferingsToXML().run()
        ExportStudentCoursesToXML().run()
        ExportStudentCourseDemandsToXML().run()
        ExportClassesToXML().run()
        ExportAcademicAreaToXML().run()
        ExportAcademicClassificationsToXML().run()
        ExportPosMajorsToXML().run()
        ExportCurriculaToXML().run()



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
import random
from uuid import uuid4

from pyoko.modelmeta import model_registry
from pyoko.conf import settings

from ulakbus.views.reports import ReporterRegistry
#from zengine.views import basic_view
from zengine.views.base import BaseView
from ulakbus.models import Personel, Ogrenci
from zengine.views.menu import Menu


class Search(BaseView):
    SEARCH_ON = None

    def __init__(self, *args, **kwargs):
        super(Search, self).__init__(*args, **kwargs)
        self.query = self.current.input['query']
        self.output['results'] = []
        self.do_search()

    def do_search(self):
        try:
            tckn = int(self.query.strip())
            objects = self.SEARCH_ON.objects.filter(tckn__startswith=tckn)
        except ValueError:
            q = self.query.replace(' ', '\ ')
            # objects = self.SEARCH_ON.objects.raw("ad:*%s* OR soyad:*%s*" % (q, q))
            objects = self.SEARCH_ON.objects.search_on('ad', 'soyad', contains=q)
        for o in objects:
            self.output['results'].append(("%s %s" % (o.ad, o.soyad), o.tckn, o.key, ''))

# @basic_view('ogrenci_ara')
class SearchStudent(Search):
    SEARCH_ON = Ogrenci


class SearchPerson(Search):
    SEARCH_ON = Personel


def get_random_msg():
    msgs = [{'type': 1, 'title': 'İşlem tamamlandı',
             'body': 'Uzun süren işlem başarıyla tamamlandı',
             'url': '#yeni_personel/?t=%s' % uuid4().hex},
            {'type': 2, 'title': 'Yeni İleti', 'body': 'Dene Mem\'den mesajınız var',
             'url': '#show_msg/?t=%s' % uuid4().hex},
            {'type': 3, 'title': 'Hata', 'body': 'Ulakbus ölümcül bir hatadan kurtarıldı!',
             'url': ''}
            ]
    return msgs[random.randrange(0, len(msgs))]


class Notification(BaseView):
    def __init__(self, current):
        super(Notification, self).__init__(current)

        # if 'read' in current.input:
        #     self.mark_as_read()
        self.output['notifications'] = []
        msg_id_to_delete = current.input.get('id')
        notifies = list(self.current.msg_cache.get_all())
        for ntf in notifies:
            if ntf['id'] == msg_id_to_delete:
                self.current.msg_cache.remove_item(ntf)
            else:
                self.output['notifications'].append(ntf)
        self.output['cmd'] = 'notification'

    # def mark_as_read(self):
    #     read_messages = self.current.input['read']
    #     for msg in read_messages:
    #         self.current.msg_cache.remove_item(msg)


class GetCurrentUser(BaseView):
    def __init__(self, current):
        super(GetCurrentUser, self).__init__(current)
        self.output['current_user'] = {}
        self.getUser(current.user)

    def getUser(self, userObject):
        currentUser = {
            "name": userObject.name,
            "surname": userObject.surname,
            "username": userObject.username,
            "roles": [{"role": role.__unicode__()} for role in userObject.role_set]
        }
        self.output['current_user'] = currentUser


class UlakbusMenu(Menu):
    def __init__(self, current):
        super(UlakbusMenu, self).__init__(current)
        self.add_reporters()
        self.add_user_data()
        self.add_settings()
        self.add_admin_crud()

    def add_admin_crud(self):
        if self.current.user.superuser:
            for mdl in model_registry.get_base_models():
                self.output['other'].append({
                    "text": mdl.Meta.verbose_name_plural,
                    "wf": 'crud',
                    "model": mdl.__name__,
                    "kategori": 'Admin'})

    def add_settings(self):
        self.output['settings'] = {
            'static_url': settings.S3_PUBLIC_URL,
        }

    def add_user_data(self):
        # add data of current logged in user
        usr = self.current.user
        role = self.current.role
        self.output['current_user'] = {
            "name": usr.name,
            "surname": usr.surname,
            "username": usr.username,
            "role": self.current.role.abstract_role.name,
            "avatar": usr.get_avatar_url(),
            "is_staff": role.is_staff,
            "is_student": role.is_student,
            "roles": [{"role": roleset.role.__unicode__()} for roleset in
                      self.current.user.role_set]
        }
        if role.is_student:
            # insert student specific data here
            self.output['current_user'].update({
            })

        elif role.is_staff:
            # insert staff specific data here
            self.output['current_user'].update({
            })

    def add_reporters(self):
        for mdl in ReporterRegistry.get_reporters():
            perm = "report.%s" % mdl['model']
            if self.current.has_permission(perm):
                self.output['other'].append(mdl)



# -*- coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Sorgu Servisi

Hitap sorgulama servislerinin kalıtılacağı
abstract HITAP Sorgula servisini içeren modül.


Attributes:
    H_USER (str): Hitap kullanıcı adı
    H_PASS (str): Hitap kullanıcı şifresi


Example:
    Servise JSON nesnesi kullanılarak istek gönderilmesi:

    .. code-block:: json

        $ curl http://localhost:11223/hizmet-okul-getir -d '{"tckn": "tckn"}'


    İsteğe dönen cevap:

    .. code-block:: json

        $ {
            'status': 'ok',
            'result': '[{
                "hazirlik": 0,
                "ogrenim_yeri": "1",
                "mezuniyet_tarihi": "07.12.2005",
                "denklik_okul": null,
                "denklik_bolum": null,
                "tckn": "tckn",
                "ogrenim_durumu": 7,
                "kayit_no": 1234567,
                "kurum_onay_tarihi": null,
                "okul_ad": "YENI MAHALLE LİSESİ",
                "bolum": "SOSYAL BİLİMLER",
                "denklik_tarihi": "01.01.1900",
                "ogrenim_suresi": 4
                },
                ...
            }]'
        }

"""

from zato.server.service import Service
from hitap_helper import HitapHelper
import os
import urllib2
from json import dumps
from six import iteritems

H_USER = os.environ["HITAP_USER"]
H_PASS = os.environ["HITAP_PASS"]


class HITAPSorgula(Service):
    """
    Hitap Sorgulama servislerinin kalıtılacağı abstract Zato servisi.

    Sorgulama servisleri gerekli girdileri (Hitap username, Hitap password, tckn)
    Hitap'a yollayıp dönecek cevabı elde edebilmektedirler.

    Attributes:
        service_name (str): İlgili Hitap sorgu servisinin adı
        bean_name (str): Hitap'tan gelen bean nesnesinin adı
        service_dict (dict): Hitap servisinden gelen cevap için sözlük.
            Cevabın içerdiği alanlarla modeldeki alanların eşlendiği sözlüğü
            ve tarih filtresi uygulanacak alanların listesini içerir.

    """

    def __init__(self):
        self.service_name = ''
        self.bean_name = ''
        self.service_dict = {}
        super(HITAPSorgula, self).__init__()

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Servise gelen istekten kimlik numarası (tckn) bilgisini alır ve
        soap outgoing bağlantısını oluşturur. Bu bilgilerle
        Hitap'a gidecek isteği hazırlayacak ve gelen cevabı elde edecek olan
        request_json fonksiyonunu çağırır.

        """

        self.logger.info("zato service started to work.")
        tckn = self.request.payload['tckn']
        conn = self.outgoing.soap['HITAP'].conn

        self.request_json(tckn, conn)

    def request_json(self, tckn, conn):
        """
        Kimlik numarası ve kullanıcı bilgileriyle birlikte Hitap'ın ilgili servisine
        istekte bulunup gelen cevabı uygun şekilde elde eder.

        Hitap'tan gelen verilerin sisteme uygun şekilde elde edilmesi için, gerekli
        Hitap sözlüğünü yaratan ve tarih alanı gibi belli alanlara uygun filteleri
        uygulayacak olan fonksiyonları çağırır.

        Veriler uygun şekilde elde edildikten sonra servis cevabı (payload) oluşturulur.

        Args:
            tckn (str): Türkiye Cumhuriyeti Kimlik Numarası
            conn (zato.outgoing.soap.conn): Zato soap outgoing bağlantısı

        Raises:
            AttributeError: İlgili servis veya bean Hitap'ta bulunmayabilir.
            urllib2.URLError: Servis yanıt vermiyor.

        """

        status = "error"
        hitap_dict = []

        try:
            # connection for hitap
            with conn.client() as client:

                if 'required_fields' in self.service_dict:
                    required_field_check = HitapHelper()
                    required_field_check.check_required_data(self.service_dict)

                # hitap response
                hitap_service = getattr(client.service, self.service_name)(H_USER, H_PASS, tckn)
                # get bean object
                service_bean = getattr(hitap_service, self.bean_name)

                self.logger.info("%s started to work." % self.service_name)
                hitap_dict = self.create_hitap_dict(service_bean, self.service_dict['fields'])

                # filtering for some fields
                if 'date_filter' in self.service_dict:
                    self.date_filter(hitap_dict)
                self.custom_filter(hitap_dict)

            status = "ok"

        except AttributeError:
            self.logger.exception("AttributeError")
            status = "error"

        except urllib2.URLError:
            self.logger.exception("Service unavailable!")
            status = "error"

        finally:
            self.response.payload = {'status': status, 'result': dumps(hitap_dict)}


    def create_hitap_dict(self, service_bean, fields):
        """
        Modeldeki alanlarla Hitap servisinden dönen verileri eşler.

        Modeldeki kayıtlarla Hitap servisindeki kayıtların alanlarının isimleri
        bir sözlük ile eşleştirilir. Böylece Hitap'tan gelen veriler
        sisteme uygun şekilde elde edilebilmektedir.

        Args:
            service_bean (obj): Hitap bean nesnesi
            fields (dict): Modeldeki alanların Hitap'taki karşılıklarını tutan map

        Returns:
            List[dict]: Hitap verisini yerele uygun biçimde tutan sözlük listesi

        """

        hitap_dict = [{k: getattr(record, v) for k, v in iteritems(fields)}
                      for record in service_bean]

        self.logger.info("hitap_dict created.")
        return hitap_dict

    def date_filter(self, hitap_dict):
        """
        Hitap sözlüğündeki tarih alanlarını yerele uygun biçime getirir.

        Hitap'ta tarih alanları için ``01.01.0001`` değeri boş değer anlamına gelirken,
        yerelde ``01.01.1900`` şeklindedir.

        Args:
            hitap_dict (List[dict]): Hitap verisini yerele uygun biçimde tutan sözlük listesi

        """

        for record in hitap_dict:
            for field in self.service_dict['date_filter']:
                record[field] = '01.01.1900' if record[field] == "01.01.0001" else record[field]

    def custom_filter(self, hitap_dict):
        """
        Hitap sözlüğüne uygulanacak ek filtreleri (varsa) gerçekleştirir.

        Gerçekleştirimi bu servisten kalıtılan servislerde yapılmaktadır.

        Args:
            hitap_dict (List[dict]): Hitap verisini yerele uygun biçimde tutan sözlük listesi

        """
        pass



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Borçlanma Sorgula

Hitap üzerinden personelin borçlanma bilgilerinin sorgulamasını yapar.

"""

from ulakbus.services.personel.hitap.hitap_sorgula import HITAPSorgula


class HizmetBorclanmaGetir(HITAPSorgula):
    """
    HITAP Sorgulama servisinden kalıtılmış Borçlanma Bilgisi Sorgulama servisi

    """

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            bean_name (str): Hitap'tan gelen bean nesnesinin adı
            service_dict (dict): Hitap servisinden gelen kayıtların alanları,
                    ``HizmetBorclanma`` modelinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları listede tutulmaktadır.

        """

        self.service_name = 'HizmetBorclanmaSorgula'
        self.bean_name = 'HizmetBorclanmaServisBean'
        self.service_dict = {
            'fields': {
                'tckn': 'tckn',
                'kayit_no': 'kayitNo',
                'ad': 'ad',
                'soyad': 'soyad',
                'emekli_sicil': 'emekliSicil',
                'derece': 'derece',
                'kademe': 'kademe',
                'ekgosterge': 'ekgosterge',
                'baslama_tarihi': 'baslamaTarihi',
                'bitis_tarihi': 'bitisTarihi',
                'gun_sayisi': 'gunSayisi',
                'kanun_kod': 'kanunKod',
                'borc_nevi': 'borcNevi',
                'toplam_tutar': 'toplamTutar',
                'odenen_miktar': 'odenenMiktar',
                'calistigi_kurum': 'calistigiKurum',
                'isyeri_il': 'isyeriIl',
                'isyeri_ilce': 'isyeriIlce',
                'borclanma_tarihi': 'borclanmaTarihi',
                'odeme_tarihi': 'odemeTarihi',
                'kurum_onay_tarihi': 'kurumOnayTarihi'
            },
            'date_filter': ['baslama_tarihi', 'bitis_tarihi', 'borclanma_tarihi',
                            'odeme_tarihi', 'kurum_onay_tarihi'],
            'required_fields': ['tckn']
        }
        super(HizmetBorclanmaGetir, self).handle()



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP IHS Sorgula

Hitap üzerinden personelin itibari hizmet süresi zammı bilgilerinin sorgulamasını yapar.

"""

from ulakbus.services.personel.hitap.hitap_sorgula import HITAPSorgula


class HizmetIHSGetir(HITAPSorgula):
    """
    HITAP Sorgulama servisinden kalıtılmış IHS Bilgisi Sorgulama servisi

    """

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            bean_name (str): Hitap'tan gelen bean nesnesinin adı
            service_dict (dict): Hitap servisinden gelen kayıtların alanları,
                    ``HizmetIHS`` modelinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları listede tutulmaktadır.

        """

        self.service_name = 'HizmetIHSSorgula'
        self.bean_name = 'HizmetIHSServisBean'
        self.service_dict = {
            'fields': {
                'tckn': 'tckn',
                'kayit_no': 'ihzID',  # TODO: ihzID mi olacak?
                'baslama_tarihi': 'baslamaTarihi',
                'bitis_tarihi': 'bitisTarihi',
                'ihz_nevi': 'ihzNevi'
            },
            'date_filter': ['baslama_tarihi', 'bitis_tarihi'],
            'required_fields': ['tckn']
        }
        super(HizmetIHSGetir, self).handle()



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Unvan Guncelle

Hitap'a personelin Unvan bilgilerinin guncellenmesini yapar.

"""

__author__ = 'H.İbrahim Yılmaz (drlinux)'

from ulakbus.services.personel.hitap.hitap_guncelle import HITAPGuncelle


class HizmetUnvanGuncelle(HITAPGuncelle):
    """
    HITAP Guncelleme servisinden kalıtılmış Hizmet Unvan Bilgi Guncelleme servisi

    """

    def handle(self):
        """Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            service_dict (dict): Request yoluyla gelen kayıtlar,
                    HizmetUnvanUpdate servisinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları ve servis tarafında gerekli olan
                    alanlar listede tutulmaktadır.

        """

        self.service_name = 'HizmetUnvanUpdate'
        self.service_dict = {
            'fields': {
                'kayitNo': self.request.payload.get('kayit_no', ''),
                'asilVekil': self.request.payload.get('asil_vekil', ''),
                'atamaSekli': self.request.payload.get('atama_sekli', ''),
                'hizmetSinifi': self.request.payload.get('hizmet_sinifi', ''),
                'tckn': self.request.payload.get('tckn', ''),
                'unvanKod': self.request.payload.get('unvan_kod', ''),
                'unvanTarihi': self.request.payload.get('unvan_tarihi', ''),
                'unvanBitisTarihi': self.request.payload.get('unvan_bitis_tarihi', ''),
                'kurumOnayTarihi': self.request.payload.get('kurum_onay_tarihi', ''),
                'fhzOrani': self.request.payload.get('fhz_orani', '')
            },
            'date_filter': ['unvanTarihi', 'unvanBitisTarihi', 'kurumOnayTarihi'],
            'required_fields': ['kayitNo', 'tckn', 'unvanKod', 'unvanTarihi', 'hizmetSinifi',
                                'asilVekil', 'atamaSekli', 'kurumOnayTarihi']
        }
        super(HizmetUnvanGuncelle, self).handle()
